@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2019, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,7 +33,7 @@ struct argp_option program_options[] =
       UI_KEY_GLOBALHDU,
       "STR",
       0,
-      "Use this HDU for all inputs, ignore `--hdu'.",
+      "Use this HDU for all inputs, ignore '--hdu'.",
       GAL_OPTIONS_GROUP_INPUT,
       &p->globalhdu,
       GAL_TYPE_STRING,
