@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -512,7 +512,8 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
 
       /* Spin off threads for each pixel. */
       gal_threads_spin_off(arithmetic_filter, &afp, afp.input->size,
-                           p->cp.numthreads);
+                           p->cp.numthreads, p->cp.minmapsize,
+                           p->cp.quietmmap);
     }
 
 
@@ -702,11 +703,145 @@ arithmetic_invert(struct arithmeticparams *p, char *token)
 
 
 
+#define INTERPOLATE_REGION(TYPE,OP,FUNC) {                              \
+    TYPE mm, b, *a=in->array, *m=minmax->array;                         \
+    FUNC(in->type, &mm);                                                \
+    gal_blank_write(&b, in->type);                                      \
+    for(i=0;i<minmax->size;++i) m[i]=mm;                                \
+    for(i=0;i<in->size;++i)                                             \
+      {                                                                 \
+        if( l[i]>0 )                                                    \
+          GAL_DIMENSION_NEIGHBOR_OP(i, in->ndim, in->dsize, in->ndim,   \
+                                    dinc,                               \
+            {                                                           \
+              if(in->type==GAL_TYPE_FLOAT32 || in->type==GAL_TYPE_FLOAT64) \
+                { if( a[nind] OP m[l[i]] ) m[l[i]]=a[nind]; }           \
+              else                                                      \
+                { if( a[nind]!=b && a[nind] OP m[l[i]] ) m[l[i]]=a[nind]; } \
+            });                                                         \
+      }                                                                 \
+    for(i=0;i<in->size;++i) if( l[i]>0 ) { a[i]=m[l[i]]; }              \
+}
+#define INTERPOLATE_REGION_OP(TYPE) {                                   \
+    switch(operator)                                                    \
+      {                                                                 \
+      case ARITHMETIC_OP_INTERPOLATE_MINOFREGION:                       \
+        INTERPOLATE_REGION(TYPE,<,gal_type_max); break;                 \
+      case ARITHMETIC_OP_INTERPOLATE_MAXOFREGION:                       \
+        INTERPOLATE_REGION(TYPE,>,gal_type_min); break;                 \
+      default:                                                          \
+        error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to " \
+              "fix the problem. The operator code %d isn't recognized", \
+              __func__, PACKAGE_BUGREPORT, operator);                   \
+      }                                                                 \
+  }
 static void
-arithmetic_interpolate(struct arithmeticparams *p, char *token)
+arithmetic_interpolate_region(struct arithmeticparams *p,
+                              int operator, char *token)
+{
+  /* Pop the dataset to interpolate. */
+  int32_t *l;
+  gal_data_t *minmax;
+  gal_data_t *lab=NULL, *flag;
+  size_t i, *con, *dinc, numlabs;
+
+  /* First pop the number of nearby neighbors.*/
+  gal_data_t *connectivity = operands_pop(p, token);
+
+  /* Then pop the actual dataset to interpolate. */
+  gal_data_t *in = operands_pop(p, token);
+
+  /* Do proper sanity checks on 'con'. */
+  if(connectivity->size!=1)
+    error(EXIT_FAILURE, 0, "the first popped operand to the "
+          "'interpolate-XXXofregion' operators must be a single number. "
+          "However, it has %zu elements", connectivity->size);
+  if( connectivity->type==GAL_TYPE_FLOAT32
+      || connectivity->type==GAL_TYPE_FLOAT64)
+    error(EXIT_FAILURE, 0, "the first popped operand to "
+          "'interpolate-XXXofregion' operators is the connectivity to "
+          "define connected blank regions (a counter, an integer, with "
+          "a maximum of the number of dimensions of the input). It must "
+          "NOT be a floating point.\n\n"
+          "If its already an integer, but in a floating point container, "
+          "you can use the 'int32' operator to convert it to a 32-bit "
+          "integer for example");
+
+  /* Convert connectivity to an integer type and make sure its not larger
+     than dimensions of the input. */
+  connectivity=gal_data_copy_to_new_type_free(connectivity,
+                                              GAL_TYPE_SIZE_T);
+  con=connectivity->array;
+  if(con[0]>in->ndim)
+    error(EXIT_FAILURE, 0, "the first popped operand to "
+          "'interpolate-XXXofregion' operators must not be larger than "
+          "the number of dimensions of the input. The connectivity is "
+          "used to define connected blank regions. For example in a "
+          "2D dataset, a connectivity of 1 corresponds to 4-connected "
+          "neighbors and connectivity 2 corresponds to 8-connected "
+          "neighbors");
+
+  /* First make sure the input has blank values. If it doesn't, don't waste
+     time, just return it. */
+  if( gal_blank_present(in, 1)==0 )
+    { operands_add(p, NULL, in); return; }
+
+  /* Build a binary image with the blank regions masked and label them,
+     then free the flagged array. */
+  flag=gal_blank_flag(in);
+  numlabs=gal_binary_connected_components(flag, &lab, con[0]);
+  gal_data_free(flag);
+
+  /* Allocate array to keep maximum values for each region. Just note that
+     because we count the regions from 1, but the indexs from 0, we'll
+     allocate one extra point. */
+  ++numlabs;
+  minmax=gal_data_alloc(NULL, in->type, 1, &numlabs, NULL, 0, in->minmapsize,
+                        in->quietmmap, NULL, NULL, NULL);
+
+  /* Parse the dataset elements for NaNs. */
+  l=lab->array;
+  dinc=gal_dimension_increment(in->ndim, in->dsize);
+  switch(in->type)
+    {
+    case GAL_TYPE_UINT8:   INTERPOLATE_REGION_OP(uint8_t);  break;
+    case GAL_TYPE_INT8:    INTERPOLATE_REGION_OP(int8_t);   break;
+    case GAL_TYPE_UINT16:  INTERPOLATE_REGION_OP(uint16_t); break;
+    case GAL_TYPE_INT16:   INTERPOLATE_REGION_OP(int16_t);  break;
+    case GAL_TYPE_UINT32:  INTERPOLATE_REGION_OP(uint32_t); break;
+    case GAL_TYPE_INT32:   INTERPOLATE_REGION_OP(int32_t);  break;
+    case GAL_TYPE_FLOAT32: INTERPOLATE_REGION_OP(float);    break;
+    case GAL_TYPE_FLOAT64: INTERPOLATE_REGION_OP(double);   break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
+            "fix the problem. The code %d is not a recognized data "
+            "type", __func__, PACKAGE_BUGREPORT, in->type);
+    }
+
+  /* For tests.
+  gal_fits_img_write(lab, "test-out.fits", NULL, NULL);
+  gal_fits_img_write(in, "test-out.fits", NULL, NULL);
+  printf("\n...%s...\n", __func__); exit(0);
+  */
+
+  /* Clean up. */
+  gal_data_free(lab);
+  gal_data_free(minmax);
+  gal_data_free(connectivity);
+
+  /* Push the interpolated dataset onto the stack. */
+  operands_add(p, NULL, in);
+}
+
+
+
+
+
+static void
+arithmetic_interpolate(struct arithmeticparams *p, int operator, char *token)
 {
-  int num_int;
   gal_data_t *interpolated;
+  int num_int, interpop=GAL_ARITHMETIC_OP_INVALID;
 
   /* First pop the number of nearby neighbors.*/
   gal_data_t *num = operands_pop(p, token);
@@ -717,12 +852,13 @@ arithmetic_interpolate(struct arithmeticparams *p, char *token)
   /* Do proper sanity checks on 'num'. */
   if(num->size!=1)
     error(EXIT_FAILURE, 0, "the first popped operand to "
-          "'interpolate-medianngb' must be a single number. However, "
-          "it has %zu elements", num->size);
+          "'interpolate-XXXXXngb' operators must be a single number. "
+          "However, it has %zu elements", num->size);
   if(num->type==GAL_TYPE_FLOAT32 || num->type==GAL_TYPE_FLOAT64)
     error(EXIT_FAILURE, 0, "the first popped operand to "
-          "'interpolate-medianngb' is the number of nearby neighbors (a "
-          "counter, an integer). It must NOT be a floating point.\n\n"
+          "'interpolate-XXXXXngb' operators is the number of nearby "
+          "neighbors (a counter, an integer). It must NOT be a floating "
+          "point.\n\n"
           "If its already an integer, but in a floating point container, "
           "you can use the 'int32' operator to convert it to a 32-bit "
           "integer for example");
@@ -731,10 +867,28 @@ arithmetic_interpolate(struct arithmeticparams *p, char *token)
   num=gal_data_copy_to_new_type_free(num, GAL_TYPE_INT32);
   num_int = *((int32_t *)(num->array));
 
+  /* Set the interpolation operator. */
+  switch(operator)
+    {
+    case ARITHMETIC_OP_INTERPOLATE_MINNGB:
+      interpop=GAL_INTERPOLATE_NEIGHBORS_FUNC_MIN;
+      break;
+    case ARITHMETIC_OP_INTERPOLATE_MAXNGB:
+      interpop=GAL_INTERPOLATE_NEIGHBORS_FUNC_MAX;
+      break;
+    case ARITHMETIC_OP_INTERPOLATE_MEDIANNGB:
+      interpop=GAL_INTERPOLATE_NEIGHBORS_FUNC_MEDIAN;
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
+            "the problem. The operator code %d isn't recognized",
+            __func__, PACKAGE_BUGREPORT, operator);
+    }
+
   /* Call the interpolation function. */
-  interpolated=gal_interpolate_close_neighbors(in, NULL, p->cp.interpmetric,
-                                               num_int, p->cp.numthreads,
-                                               1, 0);
+  interpolated=gal_interpolate_neighbors(in, NULL, p->cp.interpmetric,
+                                         num_int, p->cp.numthreads,
+                                         1, 0, interpop);
 
   /* Clean up and push the interpolated array onto the stack. */
   gal_data_free(in);
@@ -777,7 +931,7 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
           "is %ld", dim);
   if(dim > input->ndim)
     error(EXIT_FAILURE, 0, "input dataset to '%s' has %zu dimension(s), "
-          "but you have asked to collapse along dimension %zu", token,
+          "but you have asked to collapse along dimension %ld", token,
           input->ndim, dim);
 
 
@@ -866,7 +1020,7 @@ arithmetic_tofile(struct arithmeticparams *p, char *token, int freeflag)
   /* Save it to a file. */
   popped->wcs=p->refdata.wcs;
   if(popped->ndim==1 && p->onedasimage==0)
-    gal_table_write(popped, NULL, p->cp.tableformat, filename,
+    gal_table_write(popped, NULL, NULL, p->cp.tableformat, filename,
                     "ARITHMETIC", 0);
   else
     gal_fits_img_write(popped, filename, NULL, PROGRAM_NAME);
@@ -1019,8 +1173,16 @@ arithmetic_set_operator(char *string, size_t *num_operands)
         { op=ARITHMETIC_OP_FILL_HOLES;            *num_operands=0; }
       else if (!strcmp(string, "invert"))
         { op=ARITHMETIC_OP_INVERT;                *num_operands=0; }
+      else if (!strcmp(string, "interpolate-minngb"))
+        { op=ARITHMETIC_OP_INTERPOLATE_MINNGB;    *num_operands=0; }
+      else if (!strcmp(string, "interpolate-maxngb"))
+        { op=ARITHMETIC_OP_INTERPOLATE_MAXNGB;    *num_operands=0; }
       else if (!strcmp(string, "interpolate-medianngb"))
         { op=ARITHMETIC_OP_INTERPOLATE_MEDIANNGB; *num_operands=0; }
+      else if (!strcmp(string, "interpolate-minofregion"))
+        { op=ARITHMETIC_OP_INTERPOLATE_MINOFREGION; *num_operands=0; }
+      else if (!strcmp(string, "interpolate-maxofregion"))
+        { op=ARITHMETIC_OP_INTERPOLATE_MAXOFREGION; *num_operands=0; }
       else if (!strcmp(string, "collapse-sum"))
         { op=ARITHMETIC_OP_COLLAPSE_SUM;          *num_operands=0; }
       else if (!strcmp(string, "collapse-min"))
@@ -1142,8 +1304,15 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
           arithmetic_invert(p, operator_string);
           break;
 
+        case ARITHMETIC_OP_INTERPOLATE_MINNGB:
+        case ARITHMETIC_OP_INTERPOLATE_MAXNGB:
         case ARITHMETIC_OP_INTERPOLATE_MEDIANNGB:
-          arithmetic_interpolate(p, operator_string);
+          arithmetic_interpolate(p, operator, operator_string);
+          break;
+
+        case ARITHMETIC_OP_INTERPOLATE_MINOFREGION:
+        case ARITHMETIC_OP_INTERPOLATE_MAXOFREGION:
+          arithmetic_interpolate_region(p, operator, operator_string);
           break;
 
         case ARITHMETIC_OP_COLLAPSE_SUM:
@@ -1217,7 +1386,14 @@ reversepolish(struct arithmeticparams *p)
           || operands_is_name(p, token->v) )
         operands_add(p, token->v, NULL);
       else if( (data=gal_data_copy_string_to_number(token->v)) )
-        operands_add(p, NULL, data);
+        {
+          /* The 'minmapsize' and 'quietmmap' parameters should be passed
+             onto the library within numbers also (since they are the
+             only things that go in the library sometimes). */
+          data->quietmmap=p->cp.quietmmap;
+          data->minmapsize=p->cp.minmapsize;
+          operands_add(p, NULL, data);
+        }
       /* Last option is an operator: the program will abort if the token
          isn't an operator. */
       else
@@ -1290,7 +1466,7 @@ reversepolish(struct arithmeticparams *p)
          will be freed while freeing 'data'. */
       data->wcs=p->refdata.wcs;
       if(data->ndim==1 && p->onedasimage==0)
-        gal_table_write(data, NULL, p->cp.tableformat,
+        gal_table_write(data, NULL, NULL, p->cp.tableformat,
                         p->onedonstdout ? NULL : p->cp.output,
                         "ARITHMETIC", 0);
       else
