@@ -3,9 +3,9 @@ Arithmetic - Do arithmetic operations on images.
 Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -32,5 +32,10 @@ operands_add(struct arithmeticparams *p, char *filename, gal_data_t *data);
 gal_data_t *
 operands_pop(struct arithmeticparams *p, char *operator);
 
+void
+operands_set_name(struct arithmeticparams *p, char *token);
+
+int
+operands_is_name(struct arithmeticparams *p, char *token);
 
 #endif
