@@ -5,7 +5,7 @@ ConvertType is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2018, Free Software Foundation, Inc.
+Copyright (C) 2016-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -55,8 +55,8 @@ enum option_keys_enum
   UI_KEY_FLUXLOW             = 'L',
   UI_KEY_FLUXHIGH            = 'H',
   UI_KEY_MAXBYTE             = 'm',
-  UI_KEY_FLMINBYTE           = 'A',
-  UI_KEY_FHMAXBYTE           = 'B',
+  UI_KEY_FORCEMIN            = 'A',
+  UI_KEY_FORCEMAX            = 'B',
   UI_KEY_CHANGE              = 'c',
   UI_KEY_CHANGEAFTERTRUNC    = 'C',
   UI_KEY_INVERT              = 'i',
