@@ -5,7 +5,7 @@ Convolve is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2019, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -166,7 +166,7 @@ struct argp_option program_options[] =
       UI_KEY_DOMAIN,
       "STR",
       0,
-      "Convolution domain: `spatial', `frequency'.",
+      "Convolution domain: 'spatial', 'frequency'.",
       GAL_OPTIONS_GROUP_OPERATING_MODE,
       &p->domainstr,
       GAL_TYPE_STRING,
