@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018-2020, Free Software Foundation, Inc.
+Copyright (C) 2018-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -442,6 +442,7 @@ parse_objects(struct mkcatalog_passparams *pp)
   gal_data_t *xybin=NULL;
   size_t *tsize=pp->tile->dsize;
   uint8_t *u, *uf, goodvalue, *xybinarr=NULL;
+  double minima_v=FLT_MAX, maxima_v=-FLT_MAX;
   size_t d, pind=0, increment=0, num_increment=1;
   int32_t *O, *OO, *C=NULL, *objarr=p->objects->array;
   float var, sval, varval, skyval, *V=NULL, *SK=NULL, *ST=NULL;
@@ -461,15 +462,23 @@ parse_objects(struct mkcatalog_passparams *pp)
   /* If any coordinate columns are requested. */
   size_t *c = (
                /* Coordinate-related columns. */
-               ( oif[    OCOL_GX   ]
-                 || oif[ OCOL_GY   ]
-                 || oif[ OCOL_GZ   ]
-                 || oif[ OCOL_VX   ]
-                 || oif[ OCOL_VY   ]
-                 || oif[ OCOL_VZ   ]
-                 || oif[ OCOL_C_GX ]
-                 || oif[ OCOL_C_GY ]
-                 || oif[ OCOL_C_GZ ]
+               ( oif[    OCOL_GX    ]
+                 || oif[ OCOL_GY    ]
+                 || oif[ OCOL_GZ    ]
+                 || oif[ OCOL_VX    ]
+                 || oif[ OCOL_VY    ]
+                 || oif[ OCOL_VZ    ]
+                 || oif[ OCOL_C_GX  ]
+                 || oif[ OCOL_C_GY  ]
+                 || oif[ OCOL_C_GZ  ]
+                 || oif[ OCOL_MINVX ]
+                 || oif[ OCOL_MAXVX ]
+                 || oif[ OCOL_MINVY ]
+                 || oif[ OCOL_MAXVY ]
+                 || oif[ OCOL_MINVZ ]
+                 || oif[ OCOL_MAXVZ ]
+                 || oif[ OCOL_MINVNUM ]
+                 || oif[ OCOL_MAXVNUM ]
                  || sc
                  /* When the sky and its STD are tiles, we'll also need
                     the coordinate to find which tile a pixel belongs
@@ -478,7 +487,6 @@ parse_objects(struct mkcatalog_passparams *pp)
                ? gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__, "c")
                : NULL );
 
-
   /* If an XY projection area is necessary, we'll need to allocate an array
      to keep the projected space. */
   if( p->spectrum
@@ -491,7 +499,6 @@ parse_objects(struct mkcatalog_passparams *pp)
       xybinarr=xybin->array;
     }
 
-
   /* Parse each contiguous patch of memory covered by this object. */
   while( pp->start_end_inc[0] + increment <= pp->start_end_inc[1] )
     {
@@ -571,8 +578,8 @@ parse_objects(struct mkcatalog_passparams *pp)
 
                   /* General flux summations. */
                   if(xybin) xybinarr[ pind ]=2;
-                  if(oif[ OCOL_NUM    ]) oi[ OCOL_NUM     ]++;
-                  if(oif[ OCOL_SUM    ]) oi[ OCOL_SUM     ] += *V;
+                  if(oif[ OCOL_NUM ]) oi[ OCOL_NUM ]++;
+                  if(oif[ OCOL_SUM ]) oi[ OCOL_SUM ] += *V;
 
                   /* Get the necessary clump information. */
                   if(p->clumps && *C>0)
@@ -581,6 +588,50 @@ parse_objects(struct mkcatalog_passparams *pp)
                       if(oif[ OCOL_C_SUM ]) oi[ OCOL_C_SUM ] += *V;
                     }
 
+                  /* Get the extrema of the values. Note that if the minima
+                     or maxima value's coordinates are requested in any
+                     dimension, then 'OCOL_MINVNUM' or 'OCOL_MAXVNUM' will
+                     be activated). */
+                  if( oif[ OCOL_MINVNUM ] && *V<=minima_v )
+                    {
+                      /* If the value is smaller than the smallest found so
+                         far, reset the counter to one, and reset the sum
+                         of positions this one's position. */
+                      if( *V<minima_v )
+                        {
+                          minima_v = *V;
+                          oi[ OCOL_MINVNUM ]=1;
+                          if(oif[OCOL_MINVX]) oi[ OCOL_MINVX ] = c[ ndim-1 ]+1;
+                          if(oif[OCOL_MINVY]) oi[ OCOL_MINVY ] = c[ ndim-2 ]+1;
+                          if(oif[OCOL_MINVZ]) oi[ OCOL_MINVZ ] = c[ ndim-3 ]+1;
+                        }
+                      else
+                        {
+                          oi[ OCOL_MINVNUM ]++;
+                          if(oif[OCOL_MINVX]) oi[ OCOL_MINVX ] += c[ ndim-1 ]+1;
+                          if(oif[OCOL_MINVY]) oi[ OCOL_MINVY ] += c[ ndim-2 ]+1;
+                          if(oif[OCOL_MINVZ]) oi[ OCOL_MINVZ ] += c[ ndim-3 ]+1;
+                        }
+                    }
+                  if( oif[ OCOL_MAXVNUM ] && *V>=maxima_v )
+                    {
+                      if( *V>maxima_v )
+                        {
+                          maxima_v = *V;
+                          oi[ OCOL_MAXVNUM ]=1;
+                          if(oif[OCOL_MAXVX]) oi[ OCOL_MAXVX ] = c[ ndim-1 ]+1;
+                          if(oif[OCOL_MAXVY]) oi[ OCOL_MAXVY ] = c[ ndim-2 ]+1;
+                          if(oif[OCOL_MAXVZ]) oi[ OCOL_MAXVZ ] = c[ ndim-3 ]+1;
+                        }
+                      else
+                        {
+                          oi[ OCOL_MAXVNUM ]++;
+                          if(oif[OCOL_MAXVX]) oi[ OCOL_MAXVX ] += c[ ndim-1 ]+1;
+                          if(oif[OCOL_MAXVY]) oi[ OCOL_MAXVY ] += c[ ndim-2 ]+1;
+                          if(oif[OCOL_MAXVZ]) oi[ OCOL_MAXVZ ] += c[ ndim-3 ]+1;
+                        }
+                    }
+
                   /* For flux weighted centers, we can only use positive
                      values, so do those measurements here. */
                   if( *V > 0.0f )
@@ -710,6 +761,43 @@ parse_objects(struct mkcatalog_passparams *pp)
 
 
 
+/* To keep the main function easier to read. */
+static void *
+parse_init_extrema(uint8_t *cif, uint8_t type, size_t num, int max1min0)
+{
+  void *out;
+  double *out_d;
+  size_t i, *out_s;
+
+  /* Allocate the array. */
+  out=gal_pointer_allocate(type, num, 0, __func__, "out");
+
+  /* Initialize the array. */
+  switch(type)
+    {
+    case GAL_TYPE_FLOAT64:
+      out_d=out;
+      for(i=0;i<num;++i) out_d[i]= max1min0 ? -FLT_MAX : FLT_MAX;
+      break;
+    case GAL_TYPE_SIZE_T:
+      out_s=out;
+      for(i=0;i<num;++i) out_s[i]= max1min0 ? 0 : GAL_BLANK_SIZE_T;
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
+            "the problem. Type code %d isn't recognized", __func__,
+            PACKAGE_BUGREPORT, type);
+    }
+
+  /* Return the allocated array. */
+  return out;
+}
+
+
+
+
+
+
 /* Macro to help in finding the minimum and maximum coordinates. */
 #define CMIN(COL, DIM) ( ci[ CCOL_NUMALL ]==1.0f                        \
                          ? (c[ DIM ]+1)                                 \
@@ -720,9 +808,6 @@ parse_objects(struct mkcatalog_passparams *pp)
                          : ( (c[ DIM ]+1) > ci[ COL ]                   \
                              ? (c[ DIM ]+1) : ci[ COL ] ) )
 
-
-
-
 /* Parse over the clumps within an object.  */
 void
 parse_clumps(struct mkcatalog_passparams *pp)
@@ -732,8 +817,9 @@ parse_clumps(struct mkcatalog_passparams *pp)
 
   double *ci, *cir;
   gal_data_t *xybin=NULL;
-  size_t *tsize=pp->tile->dsize;
   int32_t *O, *OO, *C=NULL, nlab;
+  size_t cind, *tsize=pp->tile->dsize;
+  double *minima_v=NULL, *maxima_v=NULL;
   uint8_t *u, *uf, goodvalue, *cif=p->ciflag;
   size_t nngb=gal_dimension_num_neighbors(ndim);
   size_t i, ii, d, pind=0, increment=0, num_increment=1;
@@ -753,18 +839,26 @@ parse_clumps(struct mkcatalog_passparams *pp)
                  : NULL );
 
   /* If any coordinate columns are requested. */
-  size_t *c = ( ( cif[    CCOL_GX ]
-                  || cif[ CCOL_GY ]
-                  || cif[ CCOL_GZ ]
-                  || cif[ CCOL_VX ]
-                  || cif[ CCOL_VY ]
-                  || cif[ CCOL_VZ ]
-                  || cif[ CCOL_MINX ]
-                  || cif[ CCOL_MAXX ]
-                  || cif[ CCOL_MINY ]
-                  || cif[ CCOL_MAXY ]
-                  || cif[ CCOL_MINZ ]
-                  || cif[ CCOL_MAXZ ]
+  size_t *c = ( ( cif[    CCOL_GX    ]
+                  || cif[ CCOL_GY    ]
+                  || cif[ CCOL_GZ    ]
+                  || cif[ CCOL_VX    ]
+                  || cif[ CCOL_VY    ]
+                  || cif[ CCOL_VZ    ]
+                  || cif[ CCOL_MINX  ]
+                  || cif[ CCOL_MAXX  ]
+                  || cif[ CCOL_MINY  ]
+                  || cif[ CCOL_MAXY  ]
+                  || cif[ CCOL_MINZ  ]
+                  || cif[ CCOL_MAXZ  ]
+                  || cif[ CCOL_MINVX ]
+                  || cif[ CCOL_MAXVX ]
+                  || cif[ CCOL_MINVY ]
+                  || cif[ CCOL_MAXVY ]
+                  || cif[ CCOL_MINVZ ]
+                  || cif[ CCOL_MAXVZ ]
+                  || cif[ CCOL_MINVNUM ]
+                  || cif[ CCOL_MAXVNUM ]
                   || sc
                   || tid==GAL_BLANK_SIZE_T )
                 ? gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
@@ -792,6 +886,13 @@ parse_clumps(struct mkcatalog_passparams *pp)
                             NULL, NULL, NULL);
     }
 
+  /* For the extrema columns. */
+  if( cif[    CCOL_MINVNUM ] || cif[ CCOL_MINVX ]
+      || cif[ CCOL_MINVX   ] || cif[ CCOL_MINVZ ] )
+    minima_v=parse_init_extrema(cif, GAL_TYPE_FLOAT64, pp->clumpsinobj, 0);
+  if( cif[    CCOL_MAXVNUM ] || cif[ CCOL_MAXVX ]
+      || cif[ CCOL_MAXVY   ] || cif[ CCOL_MAXVZ ] )
+    maxima_v=parse_init_extrema(cif, GAL_TYPE_FLOAT64, pp->clumpsinobj, 1);
 
   /* Parse each contiguous patch of memory covered by this object. */
   while( pp->start_end_inc[0] + increment <= pp->start_end_inc[1] )
@@ -816,7 +917,8 @@ parse_clumps(struct mkcatalog_passparams *pp)
                 {
                   /* Pointer to make things easier. Note that the clump
                      labels start from 1, but the array indexs from 0.*/
-                  ci=&pp->ci[ (*C-1) * CCOL_NUMCOLS ];
+                  cind = *C-1;
+                  ci=&pp->ci[ cind * CCOL_NUMCOLS ];
 
                   /* Add to the area of this object. */
                   if( cif[ CCOL_NUMALL ]
@@ -825,7 +927,7 @@ parse_clumps(struct mkcatalog_passparams *pp)
                       || cif[ CCOL_MINZ ] || cif[ CCOL_MAXZ ] )
                     ci[ CCOL_NUMALL ]++;
                   if(cif[ CCOL_NUMALLXY ])
-                    ((uint8_t *)(xybin[*C-1].array))[ pind ] = 1;
+                    ((uint8_t *)(xybin[cind].array))[ pind ] = 1;
 
                   /* Raw-position related measurements. */
                   if(c)
@@ -881,7 +983,47 @@ parse_clumps(struct mkcatalog_passparams *pp)
                       if(cif[ CCOL_NUM   ]) ci[ CCOL_NUM ]++;
                       if(cif[ CCOL_SUM   ]) ci[ CCOL_SUM ] += *V;
                       if(cif[ CCOL_NUMXY ])
-                        ((uint8_t *)(xybin[*C-1].array))[ pind ] = 2;
+                        ((uint8_t *)(xybin[cind].array))[ pind ] = 2;
+
+                      /* Minimum/maximum pixel positions. */
+                      if( cif[ CCOL_MINVNUM ] && *V<=minima_v[cind] )
+                        {
+                          if( *V<minima_v[cind] )
+                            {
+                              minima_v[cind] = *V;
+                              ci[ CCOL_MINVNUM ]=1;
+                              if(cif[CCOL_MINVX]) ci[ CCOL_MINVX ] = c[ ndim-1 ]+1;
+                              if(cif[CCOL_MINVY]) ci[ CCOL_MINVY ] = c[ ndim-2 ]+1;
+                              if(cif[CCOL_MINVZ]) ci[ CCOL_MINVZ ] = c[ ndim-3 ]+1;
+                            }
+                          else
+                            {
+                              ci[ CCOL_MINVNUM ]++;
+                              if(cif[CCOL_MINVX]) ci[ CCOL_MINVX ] += c[ ndim-1 ]+1;
+                              if(cif[CCOL_MINVY]) ci[ CCOL_MINVY ] += c[ ndim-2 ]+1;
+                              if(cif[CCOL_MINVZ]) ci[ CCOL_MINVZ ] += c[ ndim-3 ]+1;
+                            }
+                        }
+                      if( cif[ CCOL_MAXVNUM ] && *V>=maxima_v[cind] )
+                        {
+                          if( *V>maxima_v[cind] )
+                            {
+                              maxima_v[cind] = *V;
+                              ci[ CCOL_MAXVNUM ]=1;
+                              if(cif[CCOL_MAXVX]) ci[ CCOL_MAXVX ] = c[ ndim-1 ]+1;
+                              if(cif[CCOL_MAXVY]) ci[ CCOL_MAXVY ] = c[ ndim-2 ]+1;
+                              if(cif[CCOL_MAXVZ]) ci[ CCOL_MAXVZ ] = c[ ndim-3 ]+1;
+                            }
+                          else
+                            {
+                              ci[ CCOL_MAXVNUM ]++;
+                              if(cif[CCOL_MAXVX]) ci[ CCOL_MAXVX ] += c[ ndim-1 ]+1;
+                              if(cif[CCOL_MAXVY]) ci[ CCOL_MAXVY ] += c[ ndim-2 ]+1;
+                              if(cif[CCOL_MAXVZ]) ci[ CCOL_MAXVZ ] += c[ ndim-3 ]+1;
+                            }
+                        }
+
+                      /* Columns that need positive values. */
                       if( *V > 0.0f )
                         {
                           if(cif[ CCOL_NUMWHT ]) ci[ CCOL_NUMWHT ]++;
@@ -1025,36 +1167,163 @@ parse_clumps(struct mkcatalog_passparams *pp)
     }
 
 
-  /* Write the projected area columns. */
-  if(xybin)
-    for(i=0;i<pp->clumpsinobj;++i)
-      {
-        /* Pointer to make things easier. */
-        ci=&pp->ci[ i * CCOL_NUMCOLS ];
+  /* Write the higher-level columns. */
+  for(i=0;i<pp->clumpsinobj;++i)
+    {
+      /* Pointer to make things easier. */
+      ci=&pp->ci[ i * CCOL_NUMCOLS ];
 
-        /* Any non-zero pixel must be set for NUMALLXY. */
-        uf=(u=xybin[i].array)+xybin[i].size;
-        do
-          if(*u)
-            {
-              if(cif[ CCOL_NUMALLXY ]          ) ci[ CCOL_NUMALLXY ]++;
-              if(cif[ CCOL_NUMXY    ] && *u==2 ) ci[ CCOL_NUMXY    ]++;
-            }
-        while(++u<uf);
+      /* Write the XY projection columns. */
+      if(xybin)
+        {
+          /* Any non-zero pixel must be set for NUMALLXY. */
+          uf=(u=xybin[i].array)+xybin[i].size;
+          do
+            if(*u)
+              {
+                if(cif[ CCOL_NUMALLXY ]          ) ci[ CCOL_NUMALLXY ]++;
+                if(cif[ CCOL_NUMXY    ] && *u==2 ) ci[ CCOL_NUMXY    ]++;
+              }
+          while(++u<uf);
 
-        /* For a check on the projected 2D areas. */
-        if(xybin && pp->object==2)
-          gal_fits_img_write(&xybin[i], "xybin.fits", NULL, NULL);
+          /* For a check on the projected 2D areas. */
+          if(xybin && pp->object==2)
+            gal_fits_img_write(&xybin[i], "xybin.fits", NULL, NULL);
 
-      }
+        }
+    }
 
 
   /* Clean up. */
-  if(c)       free(c);
-  if(sc)      free(sc);
-  if(dinc)    free(dinc);
+  if(c) free(c);
+  if(sc) free(sc);
+  if(dinc) free(dinc);
   if(ngblabs) free(ngblabs);
-  if(xybin)   gal_data_array_free(xybin, pp->clumpsinobj, 1);
+  if(minima_v) free(minima_v);
+  if(maxima_v) free(maxima_v);
+  if(xybin) gal_data_array_free(xybin, pp->clumpsinobj, 1);
+}
+
+
+
+
+
+static size_t
+parse_frac_find(gal_data_t *sorted_d, double value, double frac, int dosum)
+{
+  size_t i;
+  double check=0.0f;
+  double *sorted=sorted_d->array;
+
+  /* Parse over the sorted array and find the index. */
+  for(i=0;i<sorted_d->size;++i)
+    if(dosum)
+      { if( (check+=sorted[i]) > value*frac ) break; }
+    else
+      { if(         sorted[i]  < value*frac ) break; }
+
+  /* Return the final value. Note that if the index is zero, we should
+     actually return 1, because we are starting with the maximum. */
+  return i==0 ? 1 : i;
+}
+
+
+
+
+
+static double
+parse_frac_sum(gal_data_t *sorted_d, double value, double frac, int dosum)
+{
+  double sum=0.0f, *sorted=sorted_d->array;
+  size_t i, ind=parse_frac_find(sorted_d, value, frac, 0);
+
+  for(i=0;i<ind;++i) sum+=sorted[i];
+  return sum;
+}
+
+
+
+
+
+static void
+parse_area_of_frac_sum(struct mkcatalog_passparams *pp, gal_data_t *values,
+                       double *outarr, int o1c0)
+{
+  struct mkcatalogparams *p=pp->p;
+
+  double max, *sorted;
+  gal_data_t *sorted_d;
+  uint8_t *flag = o1c0 ? p->oiflag : p->ciflag;
+  double *fracmax = p->fracmax ? p->fracmax->array : NULL;
+  double sumlab = o1c0 ? outarr[OCOL_SUM] : outarr[CCOL_SUM];
+
+  /* Allocate the array to use. */
+  sorted_d = ( values->type==GAL_TYPE_FLOAT64
+                ? values
+                : gal_data_copy_to_new_type(values, GAL_TYPE_FLOAT64) );
+
+  /* Sort the desired labels and find the number of elements where we reach
+     half the total sum. */
+  gal_statistics_sort_decreasing(sorted_d);
+
+  /* Set the required fractions. */
+  if(flag[ o1c0 ? OCOL_HALFSUMNUM : CCOL_HALFSUMNUM ])
+    outarr[ o1c0 ? OCOL_HALFSUMNUM : CCOL_HALFSUMNUM ]
+      = parse_frac_find(sorted_d, sumlab, 0.5f, 1);
+
+  /* Values related to the maximum. */
+  if( flag[    o1c0 ? OCOL_MAXIMUM     : CCOL_MAXIMUM     ]
+      || flag[ o1c0 ? OCOL_HALFMAXNUM  : CCOL_HALFMAXNUM  ]
+      || flag[ o1c0 ? OCOL_HALFMAXSUM  : CCOL_HALFMAXSUM  ]
+      || flag[ o1c0 ? OCOL_FRACMAX1NUM : CCOL_FRACMAX1NUM ]
+      || flag[ o1c0 ? OCOL_FRACMAX1SUM : CCOL_FRACMAX1SUM ]
+      || flag[ o1c0 ? OCOL_FRACMAX2NUM : CCOL_FRACMAX2NUM ]
+      || flag[ o1c0 ? OCOL_FRACMAX2SUM : CCOL_FRACMAX2SUM ] )
+    {
+      /* Set the array and maximum value. We'll use the median of the top
+         three pixels for the maximum (to avoid noise) */
+      sorted=sorted_d->array;
+      max = ( sorted_d->size>3
+              ? (sorted[0]+sorted[1]+sorted[2])/3
+              : sorted[0] );
+
+      /* If we want the maximum value, then write it in. */
+      if(flag[ o1c0 ? OCOL_MAXIMUM : CCOL_MAXIMUM ])
+        outarr[ o1c0 ? OCOL_MAXIMUM : CCOL_MAXIMUM ] = max;
+
+      /* The number of pixels within half the maximum. */
+      if(flag[ o1c0 ? OCOL_HALFMAXNUM : CCOL_HALFMAXNUM ])
+        outarr[ o1c0 ? OCOL_HALFMAXNUM : CCOL_HALFMAXNUM ]
+          = parse_frac_find(sorted_d, max, 0.5f, 0);
+
+      /* The number of pixels within the first requested fraction of maximum */
+      if(flag[ o1c0 ? OCOL_FRACMAX1NUM : CCOL_FRACMAX1NUM ])
+        outarr[ o1c0 ? OCOL_FRACMAX1NUM : CCOL_FRACMAX1NUM ]
+          = parse_frac_find(sorted_d, max, fracmax[0], 0);
+
+      /* The number of pixels within the first requested fraction of maximum */
+      if(flag[ o1c0 ? OCOL_FRACMAX2NUM : CCOL_FRACMAX2NUM ])
+        outarr[ o1c0 ? OCOL_FRACMAX2NUM : CCOL_FRACMAX2NUM ]
+          = parse_frac_find(sorted_d, max, fracmax[1], 0);
+
+      /* The sum of the pixels within the given fraction of the maximum. */
+      if( flag[ o1c0 ? OCOL_HALFMAXSUM : CCOL_HALFMAXSUM ] )
+        outarr[ o1c0 ? OCOL_HALFMAXSUM : CCOL_HALFMAXSUM ]
+          = parse_frac_sum(sorted_d, max, 0.5f, 0);
+
+      /* Sum of the pixels within the 1st given fraction of the maximum. */
+      if( flag[ o1c0 ? OCOL_FRACMAX1SUM : CCOL_FRACMAX1SUM ] )
+        outarr[ o1c0 ? OCOL_FRACMAX1SUM : CCOL_FRACMAX1SUM ]
+          = parse_frac_sum(sorted_d, max, fracmax[0], 0);
+
+      /* Sum of the pixels within the 1st given fraction of the maximum. */
+      if( flag[ o1c0 ? OCOL_FRACMAX2SUM : CCOL_FRACMAX2SUM ] )
+        outarr[ o1c0 ? OCOL_FRACMAX2SUM : CCOL_FRACMAX2SUM ]
+          = parse_frac_sum(sorted_d, max, fracmax[1], 0);
+    }
+
+  /* Clean up and return. */
+  if(sorted_d!=values) gal_data_free(sorted_d);
 }
 
 
@@ -1082,6 +1351,12 @@ parse_order_based(struct mkcatalog_passparams *pp)
   if(tmpsize==0)
     {
       if(p->oiflag[ OCOL_MEDIAN        ]) pp->oi[ OCOL_MEDIAN       ] = NAN;
+      if(p->oiflag[ OCOL_MAXIMUM       ]) pp->oi[ OCOL_MAXIMUM      ] = NAN;
+      if(p->oiflag[ OCOL_HALFMAXSUM    ]) pp->oi[ OCOL_HALFMAXSUM   ] = NAN;
+      if(p->oiflag[ OCOL_HALFMAXNUM    ]) pp->oi[ OCOL_HALFMAXNUM   ] = 0;
+      if(p->oiflag[ OCOL_HALFSUMNUM    ]) pp->oi[ OCOL_HALFSUMNUM   ] = 0;
+      if(p->oiflag[ OCOL_FRACMAX1NUM   ]) pp->oi[ OCOL_FRACMAX1NUM  ] = 0;
+      if(p->oiflag[ OCOL_FRACMAX2NUM   ]) pp->oi[ OCOL_FRACMAX2NUM  ] = 0;
       if(p->oiflag[ OCOL_SIGCLIPNUM    ]) pp->oi[ OCOL_SIGCLIPNUM   ] = 0;
       if(p->oiflag[ OCOL_SIGCLIPSTD    ]) pp->oi[ OCOL_SIGCLIPSTD   ] = 0;
       if(p->oiflag[ OCOL_SIGCLIPMEAN   ]) pp->oi[ OCOL_SIGCLIPMEAN  ] = NAN;
@@ -1091,10 +1366,16 @@ parse_order_based(struct mkcatalog_passparams *pp)
           {
             ci=&pp->ci[ i * CCOL_NUMCOLS ];
             if(p->ciflag[ CCOL_MEDIAN        ]) ci[ CCOL_MEDIAN      ] = NAN;
+            if(p->ciflag[ CCOL_MAXIMUM       ]) ci[ CCOL_MAXIMUM     ] = NAN;
+            if(p->ciflag[ CCOL_HALFMAXSUM    ]) ci[ CCOL_HALFMAXSUM  ] = NAN;
+            if(p->ciflag[ CCOL_HALFMAXNUM    ]) ci[ CCOL_HALFMAXNUM  ] = 0;
+            if(p->ciflag[ CCOL_HALFSUMNUM    ]) ci[ CCOL_HALFSUMNUM  ] = 0;
+            if(p->ciflag[ CCOL_FRACMAX1NUM   ]) ci[ CCOL_FRACMAX1NUM ] = 0;
+            if(p->ciflag[ CCOL_FRACMAX2NUM   ]) ci[ CCOL_FRACMAX2NUM ] = 0;
             if(p->ciflag[ CCOL_SIGCLIPNUM    ]) ci[ CCOL_SIGCLIPNUM  ] = 0;
             if(p->ciflag[ CCOL_SIGCLIPSTD    ]) ci[ CCOL_SIGCLIPSTD  ] = 0;
             if(p->ciflag[ CCOL_SIGCLIPMEAN   ]) ci[ CCOL_SIGCLIPMEAN ] = NAN;
-            if(p->ciflag[ CCOL_SIGCLIPMEDIAN ]) ci[CCOL_SIGCLIPMEDIAN] = NAN;
+            if(p->ciflag[ CCOL_SIGCLIPMEDIAN ]) ci[ CCOL_SIGCLIPMEDIAN] = NAN;
           }
       return;
     }
@@ -1105,25 +1386,29 @@ parse_order_based(struct mkcatalog_passparams *pp)
                          p->cp.minmapsize, p->cp.quietmmap, NULL, NULL,
                          NULL);
 
-  /* Allocate space for the clump values. */
+  /* Clump preparations. */
   if(p->clumps)
     {
+      /* Allocate the necessary space. */
       errno=0;
       clumpsvals=malloc(pp->clumpsinobj * sizeof *clumpsvals);
       if(clumpsvals==NULL)
         error(EXIT_FAILURE, errno, "%s: couldn't allocate 'clumpsvals' for "
               "%zu clumps", __func__, pp->clumpsinobj);
 
-
       /* Allocate the array necessary to keep the values of each clump. */
       ccounter=gal_pointer_allocate(GAL_TYPE_SIZE_T, pp->clumpsinobj, 1,
                                     __func__, "ccounter");
       for(i=0;i<pp->clumpsinobj;++i)
         {
           tmpsize=pp->ci[ i * CCOL_NUMCOLS + CCOL_NUM ];
-          clumpsvals[i]=gal_data_alloc(NULL, p->values->type, 1, &tmpsize,
-                                       NULL, 0, p->cp.minmapsize,
-                                       p->cp.quietmmap, NULL, NULL, NULL);
+          clumpsvals[i] = ( tmpsize
+                            ? gal_data_alloc(NULL, p->values->type, 1,
+                                             &tmpsize, NULL, 0,
+                                             p->cp.minmapsize,
+                                             p->cp.quietmmap,
+                                             NULL, NULL, NULL)
+                            : NULL );
         }
     }
 
@@ -1151,7 +1436,7 @@ parse_order_based(struct mkcatalog_passparams *pp)
                       gal_type_sizeof(p->values->type) );
 
               /* We are also on a clump. */
-              if(p->clumps && *C>0)
+              if(p->clumps && *C>0 && clumpsvals[*C-1]!=NULL)
                 memcpy( gal_pointer_increment(clumpsvals[*C-1]->array,
                                               ccounter[*C-1]++,
                                               p->values->type), V,
@@ -1201,6 +1486,14 @@ parse_order_based(struct mkcatalog_passparams *pp)
       gal_data_free(result);
     }
 
+  /* Fractional values. */
+  if( p->oiflag[    OCOL_MAXIMUM     ]
+      || p->oiflag[ OCOL_HALFMAXNUM  ]
+      || p->oiflag[ OCOL_HALFMAXSUM  ]
+      || p->oiflag[ OCOL_HALFSUMNUM  ]
+      || p->oiflag[ OCOL_FRACMAX1NUM ]
+      || p->oiflag[ OCOL_FRACMAX2NUM ] )
+    parse_area_of_frac_sum(pp, objvals, pp->oi, 1);
 
   /* Clean up the object values. */
   gal_data_free(objvals);
@@ -1214,39 +1507,76 @@ parse_order_based(struct mkcatalog_passparams *pp)
           /* Set the main row to fill. */
           ci=&pp->ci[ i * CCOL_NUMCOLS ];
 
-          /* Do the necessary calculation. */
+          /* Median. */
           if(p->ciflag[ CCOL_MEDIAN ])
             {
-              result=gal_statistics_median(clumpsvals[i], 1);
-              result=gal_data_copy_to_new_type_free(result, GAL_TYPE_FLOAT64);
-              ci[ CCOL_MEDIAN ] = ( *((double *)(result->array))
-                                    - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]) );
-              gal_data_free(result);
+              if(clumpsvals[i])
+                {
+                  result=gal_statistics_median(clumpsvals[i], 1);
+                  result=gal_data_copy_to_new_type_free(result, GAL_TYPE_FLOAT64);
+                  ci[ CCOL_MEDIAN ] = ( *((double *)(result->array))
+                                        - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]) );
+                  gal_data_free(result);
+                }
+              else ci[ CCOL_MEDIAN ] = NAN;
             }
+
+          /* Sigma-clipping measurements. */
           if(p->ciflag[ CCOL_SIGCLIPNUM ]
              || p->ciflag[ CCOL_SIGCLIPSTD ]
              || p->ciflag[ CCOL_SIGCLIPMEAN ]
              || p->ciflag[ CCOL_SIGCLIPMEDIAN ])
             {
-              /* Calculate the sigma-clipped results and write them in any
-                 requested column. */
-              result=gal_statistics_sigma_clip(clumpsvals[i], p->sigmaclip[0],
-                                               p->sigmaclip[1], 1, 1);
-              sigcliparr=result->array;
-              if(p->ciflag[ CCOL_SIGCLIPNUM ])
-                ci[CCOL_SIGCLIPNUM]=sigcliparr[0];
-              if(p->ciflag[ CCOL_SIGCLIPSTD ])
-                ci[CCOL_SIGCLIPSTD]=( sigcliparr[3]
-                                      - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]));
-              if(p->ciflag[ CCOL_SIGCLIPMEAN ])
-                ci[CCOL_SIGCLIPMEAN]=( sigcliparr[2]
-                                       - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]));
-              if(p->ciflag[ CCOL_SIGCLIPMEDIAN ])
-                ci[CCOL_SIGCLIPMEDIAN]=( sigcliparr[1]
-                                         - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]));
+              if(clumpsvals[i])
+                {
+                  result=gal_statistics_sigma_clip(clumpsvals[i], p->sigmaclip[0],
+                                                   p->sigmaclip[1], 1, 1);
+                  sigcliparr=result->array;
+                  if(p->ciflag[ CCOL_SIGCLIPNUM ])
+                    ci[CCOL_SIGCLIPNUM]=sigcliparr[0];
+                  if(p->ciflag[ CCOL_SIGCLIPSTD ])
+                    ci[CCOL_SIGCLIPSTD]=( sigcliparr[3]
+                                          - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]));
+                  if(p->ciflag[ CCOL_SIGCLIPMEAN ])
+                    ci[CCOL_SIGCLIPMEAN]=( sigcliparr[2]
+                                           - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]));
+                  if(p->ciflag[ CCOL_SIGCLIPMEDIAN ])
+                    ci[CCOL_SIGCLIPMEDIAN]=( sigcliparr[1]
+                                             - (ci[ CCOL_RIV_SUM ]/ci[ CCOL_RIV_NUM ]));
+                  gal_data_free(result);
+                }
+              else
+                {
+                  if(p->ciflag[ CCOL_SIGCLIPNUM    ]) ci[ CCOL_SIGCLIPNUM  ]=NAN;
+                  if(p->ciflag[ CCOL_SIGCLIPSTD    ]) ci[ CCOL_SIGCLIPSTD  ]=NAN;
+                  if(p->ciflag[ CCOL_SIGCLIPMEAN   ]) ci[ CCOL_SIGCLIPMEAN ]=NAN;
+                  if(p->ciflag[ CCOL_SIGCLIPMEDIAN ]) ci[CCOL_SIGCLIPMEDIAN]=NAN;
+                }
+            }
 
-              /* Clean up the sigma-clipped values. */
-              gal_data_free(result);
+          /* Estimate half of the total sum. */
+          if( p->ciflag[    CCOL_MAXIMUM     ]
+              || p->ciflag[ CCOL_HALFMAXNUM  ]
+              || p->ciflag[ CCOL_HALFMAXSUM  ]
+              || p->ciflag[ CCOL_HALFSUMNUM  ]
+              || p->ciflag[ CCOL_FRACMAX1NUM ]
+              || p->ciflag[ CCOL_FRACMAX1SUM ]
+              || p->ciflag[ CCOL_FRACMAX2NUM ]
+              || p->ciflag[ CCOL_FRACMAX2SUM ] )
+            {
+              if(clumpsvals[i])
+                parse_area_of_frac_sum(pp, clumpsvals[i], ci, 0);
+              else
+                {
+                  if( p->ciflag[ CCOL_MAXIMUM     ]) ci[ CCOL_MAXIMUM     ]=NAN;
+                  if( p->ciflag[ CCOL_HALFMAXNUM  ]) ci[ CCOL_HALFMAXNUM  ]=NAN;
+                  if( p->ciflag[ CCOL_HALFMAXSUM  ]) ci[ CCOL_HALFMAXSUM  ]=NAN;
+                  if( p->ciflag[ CCOL_HALFSUMNUM  ]) ci[ CCOL_HALFSUMNUM  ]=NAN;
+                  if( p->ciflag[ CCOL_FRACMAX1NUM ]) ci[ CCOL_FRACMAX1NUM ]=NAN;
+                  if( p->ciflag[ CCOL_FRACMAX1SUM ]) ci[ CCOL_FRACMAX1SUM ]=NAN;
+                  if( p->ciflag[ CCOL_FRACMAX2NUM ]) ci[ CCOL_FRACMAX2NUM ]=NAN;
+                  if( p->ciflag[ CCOL_FRACMAX2SUM ]) ci[ CCOL_FRACMAX2SUM ]=NAN;
+                }
             }
 
           /* Clean up this clump's values. */
