@@ -5,7 +5,7 @@ MakeProfiles is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -171,95 +171,98 @@ saveindividual(struct mkonthread *mkp)
   /* Write profile settings into the FITS file. */
   gal_fits_key_list_add(&keys, GAL_TYPE_STRING, "PROFILE", 0,
                         ui_profile_name_write(mkp->func), 0,
-                        "Radial function", 0, NULL);
+                        "Radial function", 0, NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT64, "XCENTER", 0,
                         &p->x[id], 0, "Center of profile in catalog "
-                        "(FITS axis 1)", 0, NULL);
+                        "(FITS axis 1)", 0, NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT64, "YCENTER", 0,
                         &p->y[id], 0, "Center of profile in catalog "
-                        "(FITS axis 2)", 0, NULL);
+                        "(FITS axis 2)", 0, NULL, 0);
   if(ndim==3)
     gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT64, "ZCENTER", 0,
                           &p->z[id], 0, "Center of profile in catalog "
-                          "(FITS axis 3)", 0, NULL);
+                          "(FITS axis 3)", 0, NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "RADIUS", 0,
                         &p->r[id], 0, "Radial parameter in catalog",
-                        0, NULL);
+                        0, NULL, 0);
   if( mkp->func==PROFILE_SERSIC || mkp->func==PROFILE_MOFFAT )
     gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "PINDEX", 0,
                           &p->r[id], 0, "Index (Sersic or Moffat) of profile"
-                          "in catalog", 0, NULL);
+                          "in catalog", 0, NULL, 0);
   if(ndim==2)
     {
       gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "PA_DEG", 0,
                             &p->p1[id], 0, "Position angle of profile in "
-                            "catalog", 0, "deg");
+                            "catalog", 0, "deg", 0);
       gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "AXISRATIO", 0,
                             &p->q1[id], 0, "Axis ratio of profile in catalog",
-                            0, NULL);
+                            0, NULL, 0);
     }
   else
     {
       gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "PA1_DEG", 0,
                             &p->p1[id], 0, "First X-Z-X Euler angle in 3D", 0,
-                            "deg");
+                            "deg", 0);
       gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "PA2_DEG", 0,
                             &p->p2[id], 0, "Second X-Z-X Euler angle in 3D", 0,
-                            "deg");
+                            "deg", 0);
       gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "PA3_DEG", 0,
                             &p->p3[id], 0, "Third X-Z-X Euler angle in 3D", 0,
-                            "deg");
+                            "deg", 0);
       gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "AXISRATIO1", 0,
                             &p->q1[id], 0, "Axis ratio along second dim",
-                            0, NULL);
+                            0, NULL, 0);
       gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "AXISRATIO2", 0,
                             &p->q2[id], 0, "Axis ratio along third dim",
-                            0, NULL);
+                            0, NULL, 0);
     }
   gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "MAGNITUDE", 0,
                         &p->m[id], 0, "Magnitude of profile in catalog",
-                        0, NULL);
+                        0, NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "TRUNCATION", 0,
                         &p->t[id], 0, "Truncation of profile in catalog",
-                        0, NULL);
+                        0, NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_STRING, "RNGNAME", 0,
                         (void *)(p->rng_name), 0,
-                        "Name of random number generator", 0, NULL);
+                        "Name of random number generator", 0, NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_ULONG, "RNGSEED", 0,
                         &mkp->rng_seed, 0, "Seed of random number generator",
-                        0, NULL);
+                        0, NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_SIZE_T, "NUMRANDOM", 0,
                         &p->numrandom, 0,
-                        "Number of random points in central pixels", 0, NULL);
+                        "Number of random points in central pixels", 0,
+                        NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "TOLERANCE", 0,
                         &p->tolerance, 0,
                         "Tolerance level to stop random integration",
-                        0, NULL);
+                        0, NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_STRING, "MODE", 0,
                         p->mode==MKPROF_MODE_IMG?"img":"wcs", 0,
-                        "Coordinates in image or WCS units", 0, NULL);
+                        "Coordinates in image or WCS units", 0, NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "OVERSAMPLE", 0,
-                        &p->oversample, 0, "Oversampling factor", 0, NULL);
+                        &p->oversample, 0, "Oversampling factor", 0,
+                        NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "TUNITINP", 0,
                         &p->tunitinp, 0, "Truncation is in units of pixels, "
-                        "not radius", 0, NULL);
+                        "not radius", 0, NULL, 0);
   if( !isnan(p->zeropoint) )
       gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "ZEROPOINT", 0,
-                            &p->zeropoint, 0, "Zeropoint magnitude", 0, NULL);
+                            &p->zeropoint, 0, "Zeropoint magnitude", 0,
+                            NULL, 0);
   if( mkp->func==PROFILE_CIRCUMFERENCE )
       gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "CIRCUMWIDTH", 0,
                             &p->circumwidth, 0, "Width of circumference "
-                            "(inward) profiles", 0, NULL);
+                            "(inward) profiles", 0, NULL, 0);
   if( mkp->func==PROFILE_FLAT || mkp->func==PROFILE_CIRCUMFERENCE )
       gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "MFORFLATPIX", 0,
                             &p->mforflatpix, 0, "Magnitude is flat pixel "
-                            "value", 0, NULL);
+                            "value", 0, NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "MCOLISBRIGHTNESS", 0,
                         &p->mcolisbrightness, 0, "Catalog's magnitude is "
-                        "actually brightness", 0, NULL);
+                        "actually brightness", 0, NULL, 0);
   gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "MAGATPEAK", 0,
                         &p->magatpeak, 0, "Magnitude is for peak pixel, "
-                        "not full profile", 0, NULL);
+                        "not full profile", 0, NULL, 0);
 
   gal_fits_key_list_reverse(&keys);
   gal_fits_key_write_config(&keys, "Profile configuration", "PROFILE-CONFIG",
@@ -629,7 +632,7 @@ mkprof_write(struct mkprofparams *p)
          array. */
       if(ibq->overlaps && out)
         GAL_TILE_PO_OISET(float,float,ibq->overlap_i,ibq->overlap_m,1,0, {
-            *o  = p->replace ? ( *i==0.0f ? *o : *i ) :  (*i + *o);
+            *o  = p->replace ? ( *i>*o ? *i : *o ) :  (*i + *o);
             sum += *i;
           });
 
@@ -744,13 +747,14 @@ mkprof_write(struct mkprofparams *p)
 void
 mkprof(struct mkprofparams *p)
 {
-  int err;
-  char *tmp;
   pthread_t t;            /* Thread id not used, all are saved here. */
   pthread_attr_t attr;
   pthread_barrier_t b;
+  size_t numforprint=50;
   struct mkonthread *mkp;
+  char *tmp, *mmapname=NULL;
   gal_list_str_t *comments=NULL;
+  int err, origquiet=p->cp.quiet;
   size_t i, fi, *indexs, thrdcols;
   long *onaxes=NULL, os=p->oversample;
   size_t nb, ndim=p->ndim, nt=p->cp.numthreads;
@@ -768,7 +772,9 @@ mkprof(struct mkprofparams *p)
   /* Distribute the different profiles for different threads. Note
      that one thread is left out for writing, while nt-1 are left
      for building. */
-  gal_threads_dist_in_threads(p->num, nt, &indexs, &thrdcols);
+  mmapname=gal_threads_dist_in_threads(p->num, nt,
+                                       p->cp.minmapsize, p->cp.quietmmap,
+                                       &indexs, &thrdcols);
 
 
   /* 'onaxes' are size of the merged output image without over-sampling or
@@ -829,8 +835,27 @@ mkprof(struct mkprofparams *p)
     }
 
 
-  /* Write the created arrays into the image. */
+  /* If there are too many profiles, don't print the fact that a profile
+     has been built. */
+  if(p->num>numforprint)
+    {
+      /* Let the user know that building is ongoing. */
+      if(p->cp.quiet==0)
+        printf("  ---- Building %zu profiles... ", p->num);
+
+      /* Disable the quiet flag.*/
+      p->cp.quiet=1;
+    }
+
+
+  /* Write the created arrays into the image. Set the original quiet flag
+     and let the user know that its done. */
   mkprof_write(p);
+  if(p->num>numforprint)
+    {
+      p->cp.quiet=origquiet;
+      if(p->cp.quiet==0) printf("done.\n");
+    }
 
 
   /* Write the log file. */
@@ -857,9 +882,13 @@ mkprof(struct mkprofparams *p)
       pthread_mutex_destroy(&p->qlock);
     }
 
+  /* If a merged image was created, let the user know.... */
+  if(p->mergedimgname)
+    printf("  -- Output: %s\n", p->mergedimgname);
 
   /* Clean up. */
-  free(mkp);
-  free(indexs);
+  if(mmapname) gal_pointer_mmap_free(&mmapname, p->cp.quietmmap);
+  else         free(indexs);
   if(onaxes) free(onaxes);
+  free(mkp);
 }
