@@ -5,7 +5,7 @@ MakeProfiles is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2019, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -55,7 +55,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
  **************          Radial distance       ******************
  ****************************************************************/
 /* Set the center position of the profile in the oversampled image. Note
-   that `mkp->width' is in the non-oversampled scale. IMPORTANT: the
+   that 'mkp->width' is in the non-oversampled scale. IMPORTANT: the
    ordering is in FITS coordinate order. */
 static void
 oneprofile_center_oversampled(struct mkonthread *mkp)
@@ -92,7 +92,7 @@ oneprofile_set_coord(struct mkonthread *mkp, size_t index)
   gal_dimension_index_to_coord(index, ndim, dsize, coord_c);
 
   /* Convert these coordinates to one where the profile center is at the
-     center and the image is not over-sampled. Note that only `coord_c' is
+     center and the image is not over-sampled. Note that only 'coord_c' is
      in C order.*/
   for(i=0;i<ndim;++i)
     mkp->coord[i] = ( coord_c[ndim-i-1] - mkp->center[i] )/os;
@@ -135,7 +135,7 @@ oneprofile_r_el(struct mkonthread *mkp)
     default:
       error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
             "the problem. The value %zu is not recognized for "
-            "`mkp->p->ndim'", __func__, PACKAGE_BUGREPORT, mkp->p->ndim);
+            "'mkp->p->ndim'", __func__, PACKAGE_BUGREPORT, mkp->p->ndim);
     }
 }
 
@@ -218,7 +218,7 @@ oneprofile_randompoints(struct mkonthread *mkp)
      in checks), but since it has a very negligible cost (compared to the
      random checks above) cost, its good to reset it to help in debugging
      when necessary (avoid confusion when un-commenting the checks in
-     `oneprofile_pix_by_pix'). */
+     'oneprofile_pix_by_pix'). */
   mkp->r=r_before;
   mkp->coord[0]=coord_before[0];
   mkp->coord[1]=coord_before[1];
@@ -325,7 +325,7 @@ integ2d(struct mkonthread *mkp)
 /************       Pixel by pixel building       *************/
 /*********        Positions are in C not FITS         *********/
 /**************************************************************/
-/* `oneprofile_center_oversampled' stored the center of the profile in
+/* 'oneprofile_center_oversampled' stored the center of the profile in
    floating point coordinates. This function will convert that into a
    pixel index. */
 static size_t
@@ -335,9 +335,9 @@ oneprofile_center_pix_index(struct mkonthread *mkp)
   size_t *dsize=mkp->ibq->image->dsize;
   size_t i, coord[3], ndim=mkp->p->ndim;
 
-  /* Find the coordinates of the center point. Note `mkp->center' is in
+  /* Find the coordinates of the center point. Note 'mkp->center' is in
      FITS coordinates, while coord must be in C coordinates (to be used in
-     `gal_dimension_coord_to_index'). */
+     'gal_dimension_coord_to_index'). */
   for(i=0;i<ndim;++i)
     {
       pixfrac = modf(mkp->center[i], &intpart);
@@ -379,7 +379,7 @@ oneprofile_pix_by_pix(struct mkonthread *mkp)
   if(mkp->func==PROFILE_POINT)
     { array[p]=1; return; }
 
-  /* Allocate the `byt' array. It is used as a flag to make sure that we
+  /* Allocate the 'byt' array. It is used as a flag to make sure that we
      don't re-calculate the profile value on a pixel more than once. */
   byt = gal_pointer_allocate(GAL_TYPE_UINT8,
                              gal_dimension_total_size(ndim, dsize), 1,
@@ -585,8 +585,8 @@ oneprofile_set_prof_params(struct mkonthread *mkp)
 
     default:
       error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
-            "address the problem. The value `%zu' is not recognized for "
-            "`ndim'", __func__, PACKAGE_BUGREPORT, ndim);
+            "address the problem. The value '%zu' is not recognized for "
+            "'ndim'", __func__, PACKAGE_BUGREPORT, ndim);
     }
 
 
@@ -694,6 +694,14 @@ oneprofile_set_prof_params(struct mkonthread *mkp)
 
 
 
+    case PROFILE_CUSTOM:
+      mkp->profile          = profiles_custom_table;
+      mkp->truncr           = tp ? p->t[id] : p->t[id]*p->r[id];
+      mkp->correction       = 0;
+      break;
+
+
+
     default:
       error(EXIT_FAILURE, 0, "%s: a bug! Please contact us so we can "
             "correct this problem. The profile code %u is not recognized.",
@@ -772,8 +780,8 @@ oneprofile_make(struct mkonthread *mkp)
       mkp->ibq->accufrac /= sum;
 
       /* Correct all the profile pixels. Note that ideally, if a user wants
-         a NaN valued profile, they should use the `flat' profile with
-         `--mforflatpix', which won't need this correction. However, it
+         a NaN valued profile, they should use the 'flat' profile with
+         '--mforflatpix', which won't need this correction. However, it
          might happen that they forget the later, or the catalog might be
          generated by a script that gives a NaN value for the magnitude
          with any kind of profile. In such cases if we don't check the NaN
