@@ -5,7 +5,7 @@ Segment is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018-2019, Free Software Foundation, Inc.
+Copyright (C) 2018-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -107,6 +107,7 @@ ui_initialize_options(struct segmentparams *p,
 
 
   /* Set the necessary common parameters structure. */
+  cp->program_struct     = p;
   cp->poptions           = program_options;
   cp->program_name       = PROGRAM_NAME;
   cp->program_exec       = PROGRAM_EXEC;
@@ -149,7 +150,7 @@ ui_initialize_options(struct segmentparams *p,
 
         case GAL_OPTIONS_KEY_TABLEFORMAT:
           cp->coptions[i].mandatory=GAL_OPTIONS_MANDATORY;
-          cp->coptions[i].doc="`txt', `fits-ascii', `fits-binary'.";
+          cp->coptions[i].doc="'txt', 'fits-ascii', 'fits-binary'.";
           break;
         }
     }
@@ -165,18 +166,18 @@ parse_opt(int key, char *arg, struct argp_state *state)
 {
   struct segmentparams *p = state->input;
 
-  /* Pass `gal_options_common_params' into the child parser.  */
+  /* Pass 'ygal_options_common_params' into the child parser.  */
   state->child_inputs[0] = &p->cp;
 
   /* In case the user incorrectly uses the equal sign (for example
-     with a short format or with space in the long format, then `arg`
+     with a short format or with space in the long format, then 'arg'
      start with (if the short version was called) or be (if the long
      version was called with a space) the equal sign. So, here we
      check if the first character of arg is the equal sign, then the
      user is warned and the program is stopped: */
   if(arg && arg[0]=='=')
-    argp_error(state, "incorrect use of the equal sign (`=`). For short "
-               "options, `=` should not be used and for long options, "
+    argp_error(state, "incorrect use of the equal sign ('='). For short "
+               "options, '=' should not be used and for long options, "
                "there should be no space between the option, equal sign "
                "and value");
 
@@ -224,18 +225,18 @@ parse_opt(int key, char *arg, struct argp_state *state)
 /***************       Sanity Check         *******************/
 /**************************************************************/
 /* Read and check ONLY the options. When arguments are involved, do the
-   check in `ui_check_options_and_arguments'. */
+   check in 'ui_check_options_and_arguments'. */
 static void
 ui_read_check_only_options(struct segmentparams *p)
 {
   /* If the full area is to be used as a single detection, we can't find
      the S/N value from the un-detected regions, so the user must have
-     given the `clumpsnthresh' option. */
+     given the 'clumpsnthresh' option. */
   if( p->detectionname
       && !strcmp(p->detectionname, DETECTION_ALL)
       && isnan(p->clumpsnthresh) )
-    error(EXIT_FAILURE, 0, "`--clumpsnthresh' (`-%c') not given.\n\n"
-          "When `--detection=all' (the whole input dataset is assumed to "
+    error(EXIT_FAILURE, 0, "'--clumpsnthresh' ('-%c') not given.\n\n"
+          "When '--detection=all' (the whole input dataset is assumed to "
           "be a detection), Segment can't use the undetected pixels to find "
           "the signal-to-noise ratio of true clumps. Therefore it is "
           "mandatory to provide a signal-to-noise ratio manually",
@@ -243,19 +244,19 @@ ui_read_check_only_options(struct segmentparams *p)
 
   /* If the convolved HDU is given. */
   if(p->convolvedname && p->chdu==NULL)
-    error(EXIT_FAILURE, 0, "no value given to `--convolvedhdu'. When the "
-          "`--convolved' option is called (to specify a convolved dataset "
+    error(EXIT_FAILURE, 0, "no value given to '--convolvedhdu'. When the "
+          "'--convolved' option is called (to specify a convolved dataset "
           "and avoid convolution) it is mandatory to also specify a HDU "
           "for it");
 
   /* For the options that make tables, the table format option is
      mandatory. */
   if( p->checksn && p->cp.tableformat==0 )
-    error(EXIT_FAILURE, 0, "`--tableformat' is necessary with the "
-          "`--checksn' option.\n"
-          "Please see description for `--tableformat' after running the "
-          "following command for more information (use `SPACE' to go down "
-          "the page and `q' to return to the command-line):\n\n"
+    error(EXIT_FAILURE, 0, "'--tableformat' is necessary with the "
+          "'--checksn' option.\n"
+          "Please see description for '--tableformat' after running the "
+          "following command for more information (use 'SPACE' to go down "
+          "the page and 'q' to return to the command-line):\n\n"
           "    $ info gnuastro \"Input Output options\"");
 
   /* Kernel checks. */
@@ -268,7 +269,7 @@ ui_read_check_only_options(struct segmentparams *p)
       if( gal_fits_name_is_fits(p->kernelname) && p->khdu==NULL )
         error(EXIT_FAILURE, 0, "no HDU specified for kernel. When the "
               "kernel is a FITS file, a HDU must also be specified. You "
-              "can use the `--khdu' option and give it the HDU number "
+              "can use the '--khdu' option and give it the HDU number "
               "(starting from zero), extension name, or anything "
               "acceptable by CFITSIO");
     }
@@ -278,11 +279,11 @@ ui_read_check_only_options(struct segmentparams *p)
      (higher-is-better), not the contamination level
      (lower-is-better). This actually happened in a few cases: where we
      wanted a false detection rate of 0.0001 (a super-high value!), and
-     instead of inputing 0.9999, we mistakenly gave `--snquant' a value of
-     `0.0001'. We were thus fully confused with the output (an extremely
+     instead of inputing 0.9999, we mistakenly gave '--snquant' a value of
+     '0.0001'. We were thus fully confused with the output (an extremely
      low value) and thought its a bug, while it wasn't! */
   if(p->snquant<0.1)
-    fprintf(stderr, "\nWARNING: Value of `--snquant' (`-c') is %g. Note "
+    fprintf(stderr, "\nWARNING: Value of '--snquant' ('-c') is %g. Note "
             "that this is not a contamination rate (where lower is "
             "better), it is a purity rate (where higher is better). If you "
             "intentionally asked for such a low purity level, please "
@@ -306,8 +307,8 @@ ui_check_options_and_arguments(struct segmentparams *p)
       /* If it is FITS, a HDU is also mandatory. */
       if( gal_fits_name_is_fits(p->inputname) && p->cp.hdu==NULL )
         error(EXIT_FAILURE, 0, "no HDU specified. When the input is a FITS "
-              "file, a HDU must also be specified, you can use the `--hdu' "
-              "(`-h') option and give it the HDU number (starting from "
+              "file, a HDU must also be specified, you can use the '--hdu' "
+              "('-h') option and give it the HDU number (starting from "
               "zero), extension name, or anything acceptable by CFITSIO");
 
     }
@@ -405,14 +406,19 @@ static void
 ui_prepare_inputs(struct segmentparams *p)
 {
   int32_t *i, *ii;
-  gal_data_t *maxd, *ccin, *ccout=NULL;
+  gal_data_t *maxd, *ccin, *blankflag, *ccout=NULL;
 
   /* Read the input as a single precision floating point dataset. */
   p->input = gal_array_read_one_ch_to_type(p->inputname, p->cp.hdu,
                                            NULL, GAL_TYPE_FLOAT32,
-                                           p->cp.minmapsize);
+                                           p->cp.minmapsize, p->cp.quietmmap);
   p->input->wcs = gal_wcs_read(p->inputname, p->cp.hdu, 0, 0,
                                &p->input->nwcs);
+  p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
+                                            p->input->dsize,
+                                            p->input->wcs);
+
+  /* Set the name. */
   if(p->input->name) free(p->input->name);
   gal_checkset_allocate_copy("INPUT", &p->input->name);
 
@@ -434,34 +440,41 @@ ui_prepare_inputs(struct segmentparams *p)
       /* Read the input convolved image. */
       p->conv = gal_array_read_one_ch_to_type(p->convolvedname, p->chdu,
                                               NULL, GAL_TYPE_FLOAT32,
-                                              p->cp.minmapsize);
+                                              p->cp.minmapsize,
+                                              p->cp.quietmmap);
+      p->conv->ndim=gal_dimension_remove_extra(p->conv->ndim,
+                                               p->conv->dsize,
+                                               p->conv->wcs);
       p->conv->wcs=gal_wcs_copy(p->input->wcs);
 
       /* Make sure it is the same size as the input. */
       if( gal_dimension_is_different(p->input, p->conv) )
-        error(EXIT_FAILURE, 0, "%s (hdu %s), given to `--convolved' and "
-              "`--chdu', is not the same size as the input (%s, hdu: %s)",
+        error(EXIT_FAILURE, 0, "%s (hdu %s), given to '--convolved' and "
+              "'--chdu', is not the same size as the input (%s, hdu: %s)",
               p->convolvedname, p->chdu, p->inputname, p->cp.hdu);
     }
 
 
   /* Read the detected label image and check its size. When the user gives
-     `--detection=all', then the whole input is assumed to be a single
+     '--detection=all', then the whole input is assumed to be a single
      detection. */
   if( strcmp(p->useddetectionname, DETECTION_ALL) )
     {
       /* Read the dataset into memory. */
       p->olabel = gal_array_read_one_ch(p->useddetectionname, p->dhdu,
-                                        NULL, p->cp.minmapsize);
+                                        NULL, p->cp.minmapsize,
+                                        p->cp.quietmmap);
+      p->olabel->ndim=gal_dimension_remove_extra(p->olabel->ndim,
+                                                 p->olabel->dsize, NULL);
       if( gal_dimension_is_different(p->input, p->olabel) )
-        error(EXIT_FAILURE, 0, "`%s' (hdu: %s) and `%s' (hdu: %s) have a"
+        error(EXIT_FAILURE, 0, "'%s' (hdu: %s) and '%s' (hdu: %s) have a"
               "different dimension/size", p->useddetectionname, p->dhdu,
               p->inputname, p->cp.hdu);
 
       /* Make sure the detected labels are not floating point. */
       if(p->olabel->type==GAL_TYPE_FLOAT32
          || p->olabel->type==GAL_TYPE_FLOAT64)
-        error(EXIT_FAILURE, 0, "%s (hdu: %s) has a `%s' type. The detection "
+        error(EXIT_FAILURE, 0, "%s (hdu: %s) has a '%s' type. The detection "
               "(labeled) map must have an integer type (labels/classes can "
               "only be integers). If the pixel values are integers, but only "
               "the numerical type of the image is floating-point, you can "
@@ -471,6 +484,16 @@ ui_prepare_inputs(struct segmentparams *p)
               p->dhdu, gal_type_name(p->olabel->type, 1),
               p->useddetectionname, p->dhdu);
 
+      /* If the input has blank values, set them to blank values in the
+         labeled image too. It doesn't matter if the labeled image has
+         blank pixels that aren't blank on the input image. */
+      if(gal_blank_present(p->input, 1))
+        {
+          blankflag=gal_blank_flag(p->input);
+          gal_blank_flag_apply(p->olabel, blankflag);
+          gal_data_free(blankflag);
+        }
+
       /* Get the maximum value of the input (total number of labels if they
          are separate). If the maximum is 1 (the image is a binary image),
          then apply the connected components algorithm to separate the
@@ -502,7 +525,8 @@ ui_prepare_inputs(struct segmentparams *p)
       /* Allocate the array. */
       p->olabel=gal_data_alloc(NULL, GAL_TYPE_INT32, p->input->ndim,
                                p->input->dsize, p->input->wcs, 0,
-                               p->cp.minmapsize, NULL, NULL, NULL);
+                               p->cp.minmapsize, p->cp.quietmmap,
+                               NULL, NULL, NULL);
 
       /* Initialize it to 1. */
       ii=(i=p->olabel->array)+p->olabel->size; do *i++=1; while(i<ii);
@@ -527,8 +551,14 @@ ui_prepare_kernel(struct segmentparams *p)
   if(p->kernelname)
     {
       if( strcmp(p->kernelname, UI_NO_CONV_KERNEL_NAME) )
-        p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
-                                           p->cp.minmapsize);
+        {
+          p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
+                                             p->cp.minmapsize,
+                                             p->cp.quietmmap);
+          p->kernel->ndim=gal_dimension_remove_extra(p->kernel->ndim,
+                                                     p->kernel->dsize,
+                                                     NULL);
+        }
       else
         p->kernel=NULL;
     }
@@ -538,7 +568,7 @@ ui_prepare_kernel(struct segmentparams *p)
          allocated array. */
       p->kernel=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 2,
                                kernel_2d_dsize, NULL, 0, p->cp.minmapsize,
-                               NULL, NULL, NULL);
+                               p->cp.quietmmap, NULL, NULL, NULL);
 
       /* Now copy the staticly allocated array into it. */
       k=p->kernel->array;
@@ -600,7 +630,7 @@ ui_prepare_tiles(struct segmentparams *p)
       gal_fits_img_write(check, tl->tilecheckname, NULL, PROGRAM_NAME);
       gal_data_free(check);
 
-      /* If `continueaftercheck' hasn't been called, abort NoiseChisel. */
+      /* If 'continueaftercheck' hasn't been called, abort NoiseChisel. */
       if(!p->continueaftercheck)
         ui_abort_after_check(p, tl->tilecheckname, NULL,
                              "showing all tiles over the image");
@@ -622,15 +652,15 @@ ui_check_size(gal_data_t *base, gal_data_t *comp, size_t numtiles,
   if( gal_dimension_is_different(base, comp) && numtiles!=comp->size )
     error(EXIT_FAILURE, 0, "%s (hdu: %s): doesn't have the right size "
           "(%zu elements or pixels).\n\n"
-          "It must either be the same size as `%s' (hdu: `%s'), or "
+          "It must either be the same size as '%s' (hdu: '%s'), or "
           "it must have the same number of elements as the total "
           "number of tiles in the tessellation (%zu). In the latter "
           "case, each pixel is assumed to be a fixed value for a "
           "complete tile.\n\n"
-          "Run with `-P' to see the (tessellation) options/settings "
+          "Run with '-P' to see the (tessellation) options/settings "
           "and their values). For more information on tessellation in "
           "Gnuastro, please run the following command (use the arrow "
-          "keys for up and down and press `q' to return to the "
+          "keys for up and down and press 'q' to return to the "
           "command-line):\n\n"
           "    $ info gnuastro tessellation",
           cname, chdu, comp->size, bname, bhdu, numtiles);
@@ -640,7 +670,7 @@ ui_check_size(gal_data_t *base, gal_data_t *comp, size_t numtiles,
 
 
 
-/* Subtract `sky' from the input dataset depending on its size (it may be
+/* Subtract 'sky' from the input dataset depending on its size (it may be
    the whole array or a tile-values array).. */
 static void
 ui_subtract_sky(gal_data_t *in, gal_data_t *sky,
@@ -685,7 +715,7 @@ ui_subtract_sky(gal_data_t *in, gal_data_t *sky,
 
 
 
-/* The Sky and Sky standard deviation images can be a `oneelempertile'
+/* The Sky and Sky standard deviation images can be a 'oneelempertile'
    image (only one element/pixel for a tile). So we need to do some extra
    checks on them (after reading the tessellation). */
 static float
@@ -698,37 +728,39 @@ ui_read_std_and_sky(struct segmentparams *p)
   gal_data_t *sky, *keys=gal_data_array_calloc(3);
 
   /* See if the name used for the standard deviation is a filename or a
-     value. When the string is only a number (and nothing else), `tailptr'
-     will point to the end of the string (`\0'). When the string doesn't
+     value. When the string is only a number (and nothing else), 'tailptr'
+     will point to the end of the string ('\0'). When the string doesn't
      start with a number, it will point to the start of the
-     string. However, file names might also be things like `1_std.fits'. In
-     such cases, `strtod' will return `1.0' and `tailptr' will be
-     `_std.fits'. Thus the most robust test is to see if `tailptr' is the
+     string. However, file names might also be things like '1_std.fits'. In
+     such cases, 'strtod' will return '1.0' and 'tailptr' will be
+     '_std.fits'. Thus the most robust test is to see if 'tailptr' is the
      NULL string character. */
   tmpval=strtod(p->usedstdname, &tailptr);
   if(*tailptr=='\0')
     {
       /* Allocate the dataset to keep the value and write it in. */
       p->std=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0,
-                            -1, NULL, NULL, NULL);
+                            -1, 1, NULL, NULL, NULL);
       *(float *)(p->std->array) = tmpval;
     }
   else
     {
       /* Make sure a HDU is also given. */
       if(p->stdhdu==NULL)
-        error(EXIT_FAILURE, 0, "no value given to `--stdhdu'.\n\n"
+        error(EXIT_FAILURE, 0, "no value given to '--stdhdu'.\n\n"
               "When the Sky standard deviation is a dataset, it is mandatory "
               "specify which HDU/extension it is present in. The file can "
-              "be specified explicitly with `--std'. If not, segment will "
-              "use the file given to `--detection'. If that is also not "
+              "be specified explicitly with '--std'. If not, segment will "
+              "use the file given to '--detection'. If that is also not "
               "called, it will look into the main input file (with no "
               "option)");
 
       /* Read the STD image. */
       p->std=gal_array_read_one_ch_to_type(p->usedstdname, p->stdhdu,
                                            NULL, GAL_TYPE_FLOAT32,
-                                           p->cp.minmapsize);
+                                           p->cp.minmapsize, p->cp.quietmmap);
+      p->std->ndim=gal_dimension_remove_extra(p->std->ndim,
+                                              p->std->dsize, NULL);
 
       /* Make sure it has the correct size. */
       ui_check_size(p->input, p->std, tl->tottiles, p->inputname, p->cp.hdu,
@@ -737,12 +769,12 @@ ui_read_std_and_sky(struct segmentparams *p)
 
   /* When the Standard deviation dataset (not single value) is made by
      NoiseChisel, it puts three basic statistics of the pre-interpolation
-     distribution of standard deviations in `MEDSTD', `MINSTD' and
-     `MAXSTD'. The `MEDSTD' in particular is most important because it
+     distribution of standard deviations in 'MEDSTD', 'MINSTD' and
+     'MAXSTD'. The 'MEDSTD' in particular is most important because it
      can't be inferred after the interpolations and it can be useful in
      MakeCatalog later to give a more accurate estimate of the noise
      level. So if they are present, we will read them here and write them
-     to the STD output (which is created when `--rawoutput' is not
+     to the STD output (which is created when '--rawoutput' is not
      given). */
   if(!p->rawoutput && p->std->size>1)
     {
@@ -762,31 +794,33 @@ ui_read_std_and_sky(struct segmentparams *p)
       gal_data_array_free(keys, 3, 1);
     }
 
-  /* Similar to `--std' above. */
+  /* Similar to '--std' above. */
   if(p->skyname)
     {
       tmpval=strtod(p->skyname, &tailptr);
       if(*tailptr=='\0')
         {
           sky=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0, -1,
-                             NULL, NULL, NULL);
+                             1, NULL, NULL, NULL);
           *(float *)(sky->array) = skyval = tmpval;
         }
       else
         {
           /* Make sure a HDU is also given. */
           if(p->skyhdu==NULL)
-            error(EXIT_FAILURE, 0, "no value given to `--skyhdu'.\n\n"
+            error(EXIT_FAILURE, 0, "no value given to '--skyhdu'.\n\n"
                   "When the Sky is a dataset, it is mandatory specify "
                   "which HDU/extension it is present in. The file can be "
-                  "specified explicitly with `--sky'. If it is a single "
-                  "value, you can just pass the value to `--sky' and no "
+                  "specified explicitly with '--sky'. If it is a single "
+                  "value, you can just pass the value to '--sky' and no "
                   "HDU will be necessary");
 
           /* Read the Sky dataset. */
           sky=gal_array_read_one_ch_to_type(p->skyname, p->skyhdu,
                                             NULL, GAL_TYPE_FLOAT32,
-                                            p->cp.minmapsize);
+                                            p->cp.minmapsize, p->cp.quietmmap);
+          sky->ndim=gal_dimension_remove_extra(sky->ndim, sky->dsize,
+                                               NULL);
 
           /* Check its size. */
           ui_check_size(p->input, sky, tl->tottiles, p->inputname, p->cp.hdu,
@@ -865,9 +899,9 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct segmentparams *p)
   struct gal_options_common_params *cp=&p->cp;
 
 
-  /* Include the parameters necessary for argp from this program (`args.h')
-     and for the common options to all Gnuastro (`commonopts.h'). We want
-     to directly put the pointers to the fields in `p' and `cp', so we are
+  /* Include the parameters necessary for argp from this program ('args.h')
+     and for the common options to all Gnuastro ('commonopts.h'). We want
+     to directly put the pointers to the fields in 'p' and 'cp', so we are
      simply including the header here to not have to use long macros in
      those headers which make them hard to read and modify. This also helps
      in having a clean environment: everything in those headers is only
@@ -992,12 +1026,12 @@ ui_abort_after_check(struct segmentparams *p, char *filename,
 
   if(file2name)
     {
-      if( asprintf(&name, "`%s' and `%s'", filename, file2name)<0 )
+      if( asprintf(&name, "'%s' and '%s'", filename, file2name)<0 )
         error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
     }
   else
     {
-      if( asprintf(&name, "`%s'", filename)<0 )
+      if( asprintf(&name, "'%s'", filename)<0 )
         error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
     }
 
@@ -1009,7 +1043,7 @@ ui_abort_after_check(struct segmentparams *p, char *filename,
           "%s (%s) has been created.\n\n"
           "If you want %s to continue its processing AND save any "
           "requested check outputs, please run it again with "
-          "`--continueaftercheck'.\n"
+          "'--continueaftercheck'.\n"
           "------------------------------------------------\n",
           PROGRAM_NAME, name, description, PROGRAM_NAME);
 
