@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -26,6 +26,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <errno.h>
 #include <error.h>
 #include <stdio.h>
+#include <string.h>
 
 #include <gnuastro/txt.h>
 #include <gnuastro/wcs.h>
@@ -503,11 +504,16 @@ ui_read_check_only_options(struct statisticsparams *p)
     error(EXIT_FAILURE, 0, "'--numbins' isn't set. When the histogram or "
           "cumulative frequency plots are requested, the number of bins "
           "('--numbins') is necessary");
-  if( p->histogram2d && p->numbins2==0 )
-    error(EXIT_FAILURE, 0, "'--numbins2' isn't set. When a 2D histogram "
-          "is requested, the number of bins in the second dimension "
-          "('--numbins2') is also necessary");
-
+  if( p->histogram2d )
+    {
+      if( p->numbins2==0 )
+        error(EXIT_FAILURE, 0, "'--numbins2' isn't set. When a 2D histogram "
+              "is requested, the number of bins in the second dimension "
+              "('--numbins2') is also necessary");
+      if( strcmp(p->histogram2d,"table") && strcmp(p->histogram2d,"image") )
+        error(EXIT_FAILURE, 0, "the value to '--histogram2d' can either be "
+              "'table' or 'image'");
+    }
 
   /* If an ascii plot is requested, check if the ascii number of bins and
      height are given. */
@@ -664,6 +670,7 @@ ui_out_of_range_to_blank(struct statisticsparams *p)
       break;
     case 2:
       cond = gal_arithmetic(GAL_ARITHMETIC_OP_OR, 1, flagsor, cond_l, cond_g);
+      gal_data_free(cond_g);
       break;
     }
 
@@ -774,6 +781,9 @@ ui_read_columns_in_one(gal_list_str_t *incolumns)
       /* Add each array element to the final list of columns. */
       for(i=0;i<strs->size;++i)
         gal_list_str_add(&final, strarr[i], 1);
+
+      /* Clean up. */
+      gal_data_free(strs);
     }
 
   /* Reverse the list to be in the same order. */
@@ -1008,8 +1018,8 @@ ui_preparations(struct statisticsparams *p)
             {
               gal_checkset_writable_remove(tl->tilecheckname, 0,
                                            cp->dontdelete);
-              gal_table_write(check, NULL, NULL, cp->tableformat, tl->tilecheckname,
-                              "TABLE", 0);
+              gal_table_write(check, NULL, NULL, cp->tableformat,
+                              tl->tilecheckname, "TABLE", 0);
             }
           gal_data_free(check);
         }
@@ -1041,6 +1051,7 @@ ui_preparations(struct statisticsparams *p)
           gal_blank_remove(p->input);
           gal_blank_remove(p->input->next);
 
+          gal_data_free(flag);
           p->input->next->flag &= ~GAL_DATA_FLAG_HASBLANK ;
           p->input->next->flag |= GAL_DATA_FLAG_BLANK_CH ;
         }
@@ -1182,8 +1193,8 @@ ui_free_report(struct statisticsparams *p)
   /* Free the allocated arrays: */
   free(p->cp.hdu);
   free(p->cp.output);
-  gal_data_free(p->input);
   gal_data_free(p->reference);
+  gal_list_data_free(p->input);
   gal_list_f64_free(p->tp_args);
   gal_list_i32_free(p->singlevalue);
   gal_tile_full_free_contents(&p->cp.tl);
