@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2020, Free Software Foundation, Inc.
+Copyright (C) 2016-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -483,6 +483,22 @@ ui_outcols_add_new_to_end(struct column_pack **list)
 
 
 
+static void
+ui_outcols_free(struct column_pack *list)
+{
+  struct column_pack *tmp;
+  while(list!=NULL)
+    {
+      arithmetic_token_free(list->tokens);
+      tmp=list->next;
+      free(list);
+      list=tmp;
+    }
+}
+
+
+
+
 
 
 
@@ -556,8 +572,8 @@ ui_print_info_exit(struct tableparams *p)
 static void
 ui_columns_prepare(struct tableparams *p)
 {
-  char *c, **strarr;
   gal_data_t *strs;
+  char *c, **strarr;
   size_t i, totcalled=0;
   struct column_pack *node, *last;
   gal_list_str_t *tmp, *toread=NULL;
@@ -1225,6 +1241,8 @@ ui_free_report(struct tableparams *p)
   /* Free the allocated arrays: */
   free(p->cp.hdu);
   free(p->cp.output);
+  ui_outcols_free(p->outcols);
   gal_list_data_free(p->table);
+  gal_list_data_free(p->colmetadata);
   if(p->colarray) free(p->colarray);
 }
