@@ -5,7 +5,7 @@ Warp is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2020, Free Software Foundation, Inc.
+Copyright (C) 2016-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -350,11 +350,24 @@ ui_check_options_and_arguments(struct warpparams *p)
                                              NULL, GAL_TYPE_FLOAT64,
                                              p->cp.minmapsize,
                                              p->cp.quietmmap);
+
+      /* Read the WCS and remove one-element wide dimension(s). */
       p->input->wcs=gal_wcs_read(p->inputname, p->cp.hdu, p->hstartwcs,
                                  p->hendwcs, &p->input->nwcs);
       p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
                                                 p->input->dsize,
                                                 p->input->wcs);
+
+      /* Currently Warp only works on 2D images. */
+      if(p->input->ndim!=2)
+        error(EXIT_FAILURE, 0, "input has %zu dimensions but Warp currently "
+              "only works on 2D datasets (images).\n\n"
+              "We do plan to add 3D functionality (see "
+              "https://savannah.gnu.org/task/?15729), so please get in "
+              "touch if you need it (any further interest, support or help "
+              "would be useful)", p->input->ndim);
+
+      /* Get basic WCS information. */
       if(p->input->wcs)
         {
           p->pixelscale=gal_wcs_pixel_scale(p->input->wcs);
@@ -483,6 +496,8 @@ ui_matrix_make_align(struct warpparams *p, double *tmatrix)
      to find as 'X' and the final (aligned matrix) 'P' (which is the pixel
      scale):
 
+          X            W             P
+        ------       ------      -------
         x0  x1       w0  w1      -P0   0
         x2  x3   *   w2  w3   =   0    P1
 
@@ -541,6 +556,21 @@ ui_matrix_make_align(struct warpparams *p, double *tmatrix)
       x[3] = P[1] / w[1] / A;
       x[0] = -1 * x[1] * w[3] / w[1];
       x[2] = -1 * x[3] * w[2] / w[0];
+
+      /* When the input matrix is flipped, the diagonal elements of the
+         necessary rotation will have a different sign. */
+      if(x[0]*x[3]<0)
+        if(!p->cp.quiet)
+          error(0, 0, "%s (hdu %s): WARNING (bug #55217)! The alignment may "
+                "not be correct! This is a recognized bug, we just haven't "
+                "had enough time to fix it yet, any help or support is "
+                "most welcome.\n\n"
+                "This may be due to the East and North not being left-hand "
+                "oriented. If this is the case, you can fix it by flipping "
+                "the image along the problematic axis (with '--flip=1,0' or "
+                "'--flip=0,1') and re-running Warp with '--align'. Note "
+                "that you can't yet call '--flip' in the same command as "
+                "'--align'",  p->inputname, p->cp.hdu);
     }
 
   /* For a check:
@@ -681,6 +711,16 @@ ui_matrix_from_modular(struct warpparams *p)
           break;
 
         case UI_KEY_TRANSLATE:
+          /* Translate cannot be called with '--centeroncorner'. */
+          if(p->centeroncorner)
+            error(EXIT_FAILURE, 0, "'--translate' and '--centeroncorner' "
+                  "(which is a type of translation) cannot be called "
+                  "together. To achieve the effect of --centeroncorner, "
+                  "start the warp steps with a translation of 0.5 to move "
+                  "the coordinate center to the corner of a pixel in each "
+                  "dimension");
+
+          /* Build the translation matrix. */
           module[0]=1.0f;       module[1]=0.0f;     module[2]=v1;
           module[3]=0.0f;       module[4]=1.0f;     module[5]=v2;
           module[6]=0.0f;       module[7]=0.0f;     module[8]=1.0f;
