@@ -1,6 +1,6 @@
 /* Implement the most essential subset of POSIX 1003.1-2008 pthread.h.
 
-   Copyright (C) 2009-2019 Free Software Foundation, Inc.
+   Copyright (C) 2009-2021 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -281,7 +281,11 @@ typedef unsigned int pthread_rwlockattr_t;
 # define PTHREAD_RWLOCK_INITIALIZER \
    { PTHREAD_MUTEX_INITIALIZER, PTHREAD_COND_INITIALIZER, PTHREAD_COND_INITIALIZER, 0, 0 }
 #else
-# if !@HAVE_PTHREAD_T@
+# if @HAVE_PTHREAD_T@
+#  if !defined PTHREAD_RWLOCK_INITIALIZER && defined PTHREAD_RWLOCK_INITIALIZER_NP /* z/OS */
+#   define PTHREAD_RWLOCK_INITIALIZER PTHREAD_RWLOCK_INITIALIZER_NP
+#  endif
+# else
 #  if !GNULIB_defined_pthread_rwlock_types
 typedef int pthread_rwlock_t;
 typedef unsigned int pthread_rwlockattr_t;
@@ -445,30 +449,38 @@ typedef unsigned int pthread_barrierattr_t;
 /* =========== Thread functions =========== */
 
 #if @GNULIB_PTHREAD_THREAD@
+/* The 'restrict' qualifier on ARG is nonsense, but POSIX specifies it this way.
+   Sigh.  */
 # if @REPLACE_PTHREAD_CREATE@
 #  if !(defined __cplusplus && defined GNULIB_NAMESPACE)
 #   undef pthread_create
 #   define pthread_create rpl_pthread_create
 #  endif
 _GL_FUNCDECL_RPL (pthread_create, int,
-                  (pthread_t *threadp, const pthread_attr_t *attr,
-                   void * (*mainfunc) (void *), void *arg)
+                  (pthread_t *restrict threadp,
+                   const pthread_attr_t *restrict attr,
+                   void * (*mainfunc) (void *), void *restrict arg)
                   _GL_ARG_NONNULL ((1, 3)));
 _GL_CXXALIAS_RPL (pthread_create, int,
-                  (pthread_t *threadp, const pthread_attr_t *attr,
-                   void * (*mainfunc) (void *), void *arg));
+                  (pthread_t *restrict threadp,
+                   const pthread_attr_t *restrict attr,
+                   void * (*mainfunc) (void *), void *restrict arg));
 # else
 #  if !@HAVE_PTHREAD_CREATE@
 _GL_FUNCDECL_SYS (pthread_create, int,
-                  (pthread_t *threadp, const pthread_attr_t *attr,
-                   void * (*mainfunc) (void *), void *arg)
+                  (pthread_t *restrict threadp,
+                   const pthread_attr_t *restrict attr,
+                   void * (*mainfunc) (void *), void *restrict arg)
                   _GL_ARG_NONNULL ((1, 3)));
 #  endif
-_GL_CXXALIAS_SYS (pthread_create, int,
-                  (pthread_t *threadp, const pthread_attr_t *attr,
-                   void * (*mainfunc) (void *), void *arg));
+_GL_CXXALIAS_SYS_CAST (pthread_create, int,
+                       (pthread_t *restrict threadp,
+                        const pthread_attr_t *restrict attr,
+                        void * (*mainfunc) (void *), void *restrict arg));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_create);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_create
 # if HAVE_RAW_DECL_PTHREAD_CREATE
@@ -493,7 +505,9 @@ _GL_FUNCDECL_SYS (pthread_attr_init, int, (pthread_attr_t *attr)
 #  endif
 _GL_CXXALIAS_SYS (pthread_attr_init, int, (pthread_attr_t *attr));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_attr_init);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_attr_init
 # if HAVE_RAW_DECL_PTHREAD_ATTR_INIT
@@ -522,7 +536,9 @@ _GL_FUNCDECL_SYS (pthread_attr_getdetachstate, int,
 _GL_CXXALIAS_SYS (pthread_attr_getdetachstate, int,
                   (const pthread_attr_t *attr, int *detachstatep));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_attr_getdetachstate);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_attr_getdetachstate
 # if HAVE_RAW_DECL_PTHREAD_ATTR_GETDETACHSTATE
@@ -551,7 +567,9 @@ _GL_FUNCDECL_SYS (pthread_attr_setdetachstate, int,
 _GL_CXXALIAS_SYS (pthread_attr_setdetachstate, int,
                   (pthread_attr_t *attr, int detachstate));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_attr_setdetachstate);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_attr_setdetachstate
 # if HAVE_RAW_DECL_PTHREAD_ATTR_SETDETACHSTATE
@@ -576,7 +594,9 @@ _GL_FUNCDECL_SYS (pthread_attr_destroy, int, (pthread_attr_t *attr)
 #  endif
 _GL_CXXALIAS_SYS (pthread_attr_destroy, int, (pthread_attr_t *attr));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_attr_destroy);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_attr_destroy
 # if HAVE_RAW_DECL_PTHREAD_ATTR_DESTROY
@@ -599,7 +619,9 @@ _GL_FUNCDECL_SYS (pthread_self, pthread_t, (void) _GL_ATTRIBUTE_PURE);
 #  endif
 _GL_CXXALIAS_SYS (pthread_self, pthread_t, (void));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_self);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_self
 # if HAVE_RAW_DECL_PTHREAD_SELF
@@ -622,7 +644,9 @@ _GL_FUNCDECL_SYS (pthread_equal, int, (pthread_t thread1, pthread_t thread2));
 #  endif
 _GL_CXXALIAS_SYS (pthread_equal, int, (pthread_t thread1, pthread_t thread2));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_equal);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_equal
 # if HAVE_RAW_DECL_PTHREAD_EQUAL
@@ -645,7 +669,9 @@ _GL_FUNCDECL_SYS (pthread_detach, int, (pthread_t thread));
 #  endif
 _GL_CXXALIAS_SYS (pthread_detach, int, (pthread_t thread));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_detach);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_detach
 # if HAVE_RAW_DECL_PTHREAD_DETACH
@@ -668,7 +694,9 @@ _GL_FUNCDECL_SYS (pthread_join, int, (pthread_t thread, void **valuep));
 #  endif
 _GL_CXXALIAS_SYS (pthread_join, int, (pthread_t thread, void **valuep));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_join);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_join
 # if HAVE_RAW_DECL_PTHREAD_JOIN
@@ -684,14 +712,17 @@ _GL_WARN_ON_USE (pthread_join, "pthread_join is not portable - "
 #   define pthread_exit rpl_pthread_exit
 #  endif
 _GL_FUNCDECL_RPL (pthread_exit, _Noreturn void, (void *value));
-_GL_CXXALIAS_RPL (pthread_exit, _Noreturn void, (void *value));
+_GL_CXXALIAS_RPL (pthread_exit, void, (void *value));
 # else
 #  if !@HAVE_PTHREAD_EXIT@
 _GL_FUNCDECL_SYS (pthread_exit, _Noreturn void, (void *value));
 #  endif
-_GL_CXXALIAS_SYS (pthread_exit, _Noreturn void, (void *value));
+/* Need to cast because of AIX with xlclang++.  */
+_GL_CXXALIAS_SYS_CAST (pthread_exit, void, (void *value));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_exit);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_exit
 # if HAVE_RAW_DECL_PTHREAD_EXIT
@@ -719,10 +750,13 @@ _GL_FUNCDECL_SYS (pthread_once, int,
                   (pthread_once_t *once_control, void (*initfunction) (void))
                    _GL_ARG_NONNULL ((1, 2)));
 #  endif
-_GL_CXXALIAS_SYS (pthread_once, int,
-                  (pthread_once_t *once_control, void (*initfunction) (void)));
+_GL_CXXALIAS_SYS_CAST (pthread_once, int,
+                       (pthread_once_t *once_control,
+                        void (*initfunction) (void)));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_once);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_once
 # if HAVE_RAW_DECL_PTHREAD_ONCE
@@ -740,20 +774,26 @@ _GL_WARN_ON_USE (pthread_once, "pthread_once is not portable - "
 #   define pthread_mutex_init rpl_pthread_mutex_init
 #  endif
 _GL_FUNCDECL_RPL (pthread_mutex_init, int,
-                  (pthread_mutex_t *mutex, const pthread_mutexattr_t *attr)
+                  (pthread_mutex_t *restrict mutex,
+                   const pthread_mutexattr_t *restrict attr)
                   _GL_ARG_NONNULL ((1)));
 _GL_CXXALIAS_RPL (pthread_mutex_init, int,
-                  (pthread_mutex_t *mutex, const pthread_mutexattr_t *attr));
+                  (pthread_mutex_t *restrict mutex,
+                   const pthread_mutexattr_t *restrict attr));
 # else
 #  if !@HAVE_PTHREAD_MUTEX_INIT@
 _GL_FUNCDECL_SYS (pthread_mutex_init, int,
-                  (pthread_mutex_t *mutex, const pthread_mutexattr_t *attr)
+                  (pthread_mutex_t *restrict mutex,
+                   const pthread_mutexattr_t *restrict attr)
                   _GL_ARG_NONNULL ((1)));
 #  endif
 _GL_CXXALIAS_SYS (pthread_mutex_init, int,
-                  (pthread_mutex_t *mutex, const pthread_mutexattr_t *attr));
+                  (pthread_mutex_t *restrict mutex,
+                   const pthread_mutexattr_t *restrict attr));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutex_init);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_mutex_init
 # if HAVE_RAW_DECL_PTHREAD_MUTEX_INIT
@@ -778,7 +818,9 @@ _GL_FUNCDECL_SYS (pthread_mutexattr_init, int, (pthread_mutexattr_t *attr)
 #  endif
 _GL_CXXALIAS_SYS (pthread_mutexattr_init, int, (pthread_mutexattr_t *attr));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutexattr_init);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_mutexattr_init
 # if HAVE_RAW_DECL_PTHREAD_MUTEXATTR_INIT
@@ -794,22 +836,28 @@ _GL_WARN_ON_USE (pthread_mutexattr_init, "pthread_mutexattr_init is not portable
 #   define pthread_mutexattr_gettype rpl_pthread_mutexattr_gettype
 #  endif
 _GL_FUNCDECL_RPL (pthread_mutexattr_gettype, int,
-                  (const pthread_mutexattr_t *attr, int *typep)
+                  (const pthread_mutexattr_t *restrict attr,
+                   int *restrict typep)
                   _GL_ARG_NONNULL ((1, 2)));
 _GL_CXXALIAS_RPL (pthread_mutexattr_gettype, int,
-                  (const pthread_mutexattr_t *attr, int *typep));
+                  (const pthread_mutexattr_t *restrict attr,
+                   int *restrict typep));
 # else
 #  if !@HAVE_PTHREAD_MUTEXATTR_GETTYPE@
 _GL_FUNCDECL_SYS (pthread_mutexattr_gettype, int,
-                  (const pthread_mutexattr_t *attr, int *typep)
+                  (const pthread_mutexattr_t *restrict attr,
+                   int *restrict typep)
                   _GL_ARG_NONNULL ((1, 2)));
 #  endif
 /* Need to cast, because on FreeBSD the first parameter is
                         pthread_mutexattr_t *attr.  */
 _GL_CXXALIAS_SYS_CAST (pthread_mutexattr_gettype, int,
-                       (const pthread_mutexattr_t *attr, int *typep));
+                       (const pthread_mutexattr_t *restrict attr,
+                        int *restrict typep));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutexattr_gettype);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_mutexattr_gettype
 # if HAVE_RAW_DECL_PTHREAD_MUTEXATTR_GETTYPE
@@ -836,7 +884,9 @@ _GL_FUNCDECL_SYS (pthread_mutexattr_settype, int,
 _GL_CXXALIAS_SYS (pthread_mutexattr_settype, int,
                   (pthread_mutexattr_t *attr, int type));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutexattr_settype);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_mutexattr_settype
 # if HAVE_RAW_DECL_PTHREAD_MUTEXATTR_SETTYPE
@@ -852,20 +902,24 @@ _GL_WARN_ON_USE (pthread_mutexattr_settype, "pthread_mutexattr_settype is not po
 #   define pthread_mutexattr_getrobust rpl_pthread_mutexattr_getrobust
 #  endif
 _GL_FUNCDECL_RPL (pthread_mutexattr_getrobust, int,
-                  (const pthread_mutexattr_t *attr, int *robustp)
+                  (const pthread_mutexattr_t *restrict attr,
+                   int *restrict robustp)
                   _GL_ARG_NONNULL ((1, 2)));
 _GL_CXXALIAS_RPL (pthread_mutexattr_getrobust, int,
-                  (const pthread_mutexattr_t *attr, int *robustp));
+                  (const pthread_mutexattr_t *restrict attr,
+                   int *restrict robustp));
 # else
 #  if !@HAVE_PTHREAD_MUTEXATTR_GETROBUST@
 _GL_FUNCDECL_SYS (pthread_mutexattr_getrobust, int,
-                  (const pthread_mutexattr_t *attr, int *robustp)
+                  (const pthread_mutexattr_t *restrict attr,
+                   int *restrict robustp)
                   _GL_ARG_NONNULL ((1, 2)));
 #  endif
 /* Need to cast, because on FreeBSD the first parameter is
                         pthread_mutexattr_t *attr.  */
 _GL_CXXALIAS_SYS_CAST (pthread_mutexattr_getrobust, int,
-                       (const pthread_mutexattr_t *attr, int *robustp));
+                       (const pthread_mutexattr_t *restrict attr,
+                        int *restrict robustp));
 # endif
 # if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutexattr_getrobust);
@@ -925,7 +979,9 @@ _GL_FUNCDECL_SYS (pthread_mutexattr_destroy, int, (pthread_mutexattr_t *attr)
 #  endif
 _GL_CXXALIAS_SYS (pthread_mutexattr_destroy, int, (pthread_mutexattr_t *attr));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutexattr_destroy);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_mutexattr_destroy
 # if HAVE_RAW_DECL_PTHREAD_MUTEXATTR_DESTROY
@@ -977,7 +1033,9 @@ _GL_FUNCDECL_SYS (pthread_mutex_trylock, int, (pthread_mutex_t *mutex)
 #  endif
 _GL_CXXALIAS_SYS (pthread_mutex_trylock, int, (pthread_mutex_t *mutex));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutex_trylock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_mutex_trylock
 # if HAVE_RAW_DECL_PTHREAD_MUTEX_TRYLOCK
@@ -993,20 +1051,26 @@ _GL_WARN_ON_USE (pthread_mutex_trylock, "pthread_mutex_trylock is not portable -
 #   define pthread_mutex_timedlock rpl_pthread_mutex_timedlock
 #  endif
 _GL_FUNCDECL_RPL (pthread_mutex_timedlock, int,
-                  (pthread_mutex_t *mutex, const struct timespec *abstime)
+                  (pthread_mutex_t *restrict mutex,
+                   const struct timespec *restrict abstime)
                   _GL_ARG_NONNULL ((1, 2)));
 _GL_CXXALIAS_RPL (pthread_mutex_timedlock, int,
-                  (pthread_mutex_t *mutex, const struct timespec *abstime));
+                  (pthread_mutex_t *restrict mutex,
+                   const struct timespec *restrict abstime));
 # else
 #  if !@HAVE_PTHREAD_MUTEX_TIMEDLOCK@
 _GL_FUNCDECL_SYS (pthread_mutex_timedlock, int,
-                  (pthread_mutex_t *mutex, const struct timespec *abstime)
+                  (pthread_mutex_t *restrict mutex,
+                   const struct timespec *restrict abstime)
                   _GL_ARG_NONNULL ((1, 2)));
 #  endif
 _GL_CXXALIAS_SYS (pthread_mutex_timedlock, int,
-                  (pthread_mutex_t *mutex, const struct timespec *abstime));
+                  (pthread_mutex_t *restrict mutex,
+                   const struct timespec *restrict abstime));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutex_timedlock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_mutex_timedlock
 # if HAVE_RAW_DECL_PTHREAD_MUTEX_TIMEDLOCK
@@ -1031,7 +1095,9 @@ _GL_FUNCDECL_SYS (pthread_mutex_unlock, int, (pthread_mutex_t *mutex)
 #  endif
 _GL_CXXALIAS_SYS (pthread_mutex_unlock, int, (pthread_mutex_t *mutex));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutex_unlock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_mutex_unlock
 # if HAVE_RAW_DECL_PTHREAD_MUTEX_UNLOCK
@@ -1056,7 +1122,9 @@ _GL_FUNCDECL_SYS (pthread_mutex_destroy, int, (pthread_mutex_t *mutex)
 #  endif
 _GL_CXXALIAS_SYS (pthread_mutex_destroy, int, (pthread_mutex_t *mutex));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutex_destroy);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_mutex_destroy
 # if HAVE_RAW_DECL_PTHREAD_MUTEX_DESTROY
@@ -1074,20 +1142,26 @@ _GL_WARN_ON_USE (pthread_mutex_destroy, "pthread_mutex_destroy is not portable -
 #   define pthread_rwlock_init rpl_pthread_rwlock_init
 #  endif
 _GL_FUNCDECL_RPL (pthread_rwlock_init, int,
-                  (pthread_rwlock_t *lock, const pthread_rwlockattr_t *attr)
+                  (pthread_rwlock_t *restrict lock,
+                   const pthread_rwlockattr_t *restrict attr)
                   _GL_ARG_NONNULL ((1)));
 _GL_CXXALIAS_RPL (pthread_rwlock_init, int,
-                  (pthread_rwlock_t *lock, const pthread_rwlockattr_t *attr));
+                  (pthread_rwlock_t *restrict lock,
+                   const pthread_rwlockattr_t *restrict attr));
 # else
 #  if !@HAVE_PTHREAD_RWLOCK_INIT@
 _GL_FUNCDECL_SYS (pthread_rwlock_init, int,
-                  (pthread_rwlock_t *lock, const pthread_rwlockattr_t *attr)
+                  (pthread_rwlock_t *restrict lock,
+                   const pthread_rwlockattr_t *restrict attr)
                   _GL_ARG_NONNULL ((1)));
 #  endif
 _GL_CXXALIAS_SYS (pthread_rwlock_init, int,
-                  (pthread_rwlock_t *lock, const pthread_rwlockattr_t *attr));
+                  (pthread_rwlock_t *restrict lock,
+                   const pthread_rwlockattr_t *restrict attr));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_rwlock_init);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_rwlock_init
 # if HAVE_RAW_DECL_PTHREAD_RWLOCK_INIT
@@ -1112,7 +1186,9 @@ _GL_FUNCDECL_SYS (pthread_rwlockattr_init, int, (pthread_rwlockattr_t *attr)
 #  endif
 _GL_CXXALIAS_SYS (pthread_rwlockattr_init, int, (pthread_rwlockattr_t *attr));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_rwlockattr_init);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_rwlockattr_init
 # if HAVE_RAW_DECL_PTHREAD_RWLOCKATTR_INIT
@@ -1139,7 +1215,9 @@ _GL_FUNCDECL_SYS (pthread_rwlockattr_destroy, int,
 _GL_CXXALIAS_SYS (pthread_rwlockattr_destroy, int,
                   (pthread_rwlockattr_t *attr));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_rwlockattr_destroy);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_rwlockattr_destroy
 # if HAVE_RAW_DECL_PTHREAD_RWLOCKATTR_DESTROY
@@ -1164,7 +1242,9 @@ _GL_FUNCDECL_SYS (pthread_rwlock_rdlock, int, (pthread_rwlock_t *lock)
 #  endif
 _GL_CXXALIAS_SYS (pthread_rwlock_rdlock, int, (pthread_rwlock_t *lock));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_rwlock_rdlock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_rwlock_rdlock
 # if HAVE_RAW_DECL_PTHREAD_RWLOCK_RDLOCK
@@ -1189,7 +1269,9 @@ _GL_FUNCDECL_SYS (pthread_rwlock_wrlock, int, (pthread_rwlock_t *lock)
 #  endif
 _GL_CXXALIAS_SYS (pthread_rwlock_wrlock, int, (pthread_rwlock_t *lock));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_rwlock_wrlock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_rwlock_wrlock
 # if HAVE_RAW_DECL_PTHREAD_RWLOCK_WRLOCK
@@ -1214,7 +1296,9 @@ _GL_FUNCDECL_SYS (pthread_rwlock_tryrdlock, int, (pthread_rwlock_t *lock)
 #  endif
 _GL_CXXALIAS_SYS (pthread_rwlock_tryrdlock, int, (pthread_rwlock_t *lock));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_rwlock_tryrdlock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_rwlock_tryrdlock
 # if HAVE_RAW_DECL_PTHREAD_RWLOCK_TRYRDLOCK
@@ -1239,7 +1323,9 @@ _GL_FUNCDECL_SYS (pthread_rwlock_trywrlock, int, (pthread_rwlock_t *lock)
 #  endif
 _GL_CXXALIAS_SYS (pthread_rwlock_trywrlock, int, (pthread_rwlock_t *lock));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_rwlock_trywrlock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_rwlock_trywrlock
 # if HAVE_RAW_DECL_PTHREAD_RWLOCK_TRYWRLOCK
@@ -1255,20 +1341,26 @@ _GL_WARN_ON_USE (pthread_rwlock_trywrlock, "pthread_rwlock_trywrlock is not port
 #   define pthread_rwlock_timedrdlock rpl_pthread_rwlock_timedrdlock
 #  endif
 _GL_FUNCDECL_RPL (pthread_rwlock_timedrdlock, int,
-                  (pthread_rwlock_t *lock, const struct timespec *abstime)
+                  (pthread_rwlock_t *restrict lock,
+                   const struct timespec *restrict abstime)
                   _GL_ARG_NONNULL ((1, 2)));
 _GL_CXXALIAS_RPL (pthread_rwlock_timedrdlock, int,
-                  (pthread_rwlock_t *lock, const struct timespec *abstime));
+                  (pthread_rwlock_t *restrict lock,
+                   const struct timespec *restrict abstime));
 # else
 #  if !@HAVE_PTHREAD_RWLOCK_TIMEDRDLOCK@
 _GL_FUNCDECL_SYS (pthread_rwlock_timedrdlock, int,
-                  (pthread_rwlock_t *lock, const struct timespec *abstime)
+                  (pthread_rwlock_t *restrict lock,
+                   const struct timespec *restrict abstime)
                   _GL_ARG_NONNULL ((1, 2)));
 #  endif
 _GL_CXXALIAS_SYS (pthread_rwlock_timedrdlock, int,
-                  (pthread_rwlock_t *lock, const struct timespec *abstime));
+                  (pthread_rwlock_t *restrict lock,
+                   const struct timespec *restrict abstime));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_rwlock_timedrdlock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_rwlock_timedrdlock
 # if HAVE_RAW_DECL_PTHREAD_RWLOCK_TIMEDRDLOCK
@@ -1284,20 +1376,26 @@ _GL_WARN_ON_USE (pthread_rwlock_timedrdlock, "pthread_rwlock_timedrdlock is not
 #   define pthread_rwlock_timedwrlock rpl_pthread_rwlock_timedwrlock
 #  endif
 _GL_FUNCDECL_RPL (pthread_rwlock_timedwrlock, int,
-                  (pthread_rwlock_t *lock, const struct timespec *abstime)
+                  (pthread_rwlock_t *restrict lock,
+                   const struct timespec *restrict abstime)
                   _GL_ARG_NONNULL ((1, 2)));
 _GL_CXXALIAS_RPL (pthread_rwlock_timedwrlock, int,
-                  (pthread_rwlock_t *lock, const struct timespec *abstime));
+                  (pthread_rwlock_t *restrict lock,
+                   const struct timespec *restrict abstime));
 # else
 #  if !@HAVE_PTHREAD_RWLOCK_TIMEDWRLOCK@
 _GL_FUNCDECL_SYS (pthread_rwlock_timedwrlock, int,
-                  (pthread_rwlock_t *lock, const struct timespec *abstime)
+                  (pthread_rwlock_t *restrict lock,
+                   const struct timespec *restrict abstime)
                   _GL_ARG_NONNULL ((1, 2)));
 #  endif
 _GL_CXXALIAS_SYS (pthread_rwlock_timedwrlock, int,
-                  (pthread_rwlock_t *lock, const struct timespec *abstime));
+                  (pthread_rwlock_t *restrict lock,
+                   const struct timespec *restrict abstime));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_rwlock_timedwrlock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_rwlock_timedwrlock
 # if HAVE_RAW_DECL_PTHREAD_RWLOCK_TIMEDWRLOCK
@@ -1322,7 +1420,9 @@ _GL_FUNCDECL_SYS (pthread_rwlock_unlock, int, (pthread_rwlock_t *lock)
 #  endif
 _GL_CXXALIAS_SYS (pthread_rwlock_unlock, int, (pthread_rwlock_t *lock));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_rwlock_unlock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_rwlock_unlock
 # if HAVE_RAW_DECL_PTHREAD_RWLOCK_UNLOCK
@@ -1347,7 +1447,9 @@ _GL_FUNCDECL_SYS (pthread_rwlock_destroy, int, (pthread_rwlock_t *lock)
 #  endif
 _GL_CXXALIAS_SYS (pthread_rwlock_destroy, int, (pthread_rwlock_t *lock));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_rwlock_destroy);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_rwlock_destroy
 # if HAVE_RAW_DECL_PTHREAD_RWLOCK_DESTROY
@@ -1365,20 +1467,26 @@ _GL_WARN_ON_USE (pthread_rwlock_destroy, "pthread_rwlock_destroy is not portable
 #   define pthread_cond_init rpl_pthread_cond_init
 #  endif
 _GL_FUNCDECL_RPL (pthread_cond_init, int,
-                  (pthread_cond_t *cond, const pthread_condattr_t *attr)
+                  (pthread_cond_t *restrict cond,
+                   const pthread_condattr_t *restrict attr)
                   _GL_ARG_NONNULL ((1)));
 _GL_CXXALIAS_RPL (pthread_cond_init, int,
-                  (pthread_cond_t *cond, const pthread_condattr_t *attr));
+                  (pthread_cond_t *restrict cond,
+                   const pthread_condattr_t *restrict attr));
 # else
 #  if !@HAVE_PTHREAD_COND_INIT@
 _GL_FUNCDECL_SYS (pthread_cond_init, int,
-                  (pthread_cond_t *cond, const pthread_condattr_t *attr)
+                  (pthread_cond_t *restrict cond,
+                   const pthread_condattr_t *restrict attr)
                   _GL_ARG_NONNULL ((1)));
 #  endif
 _GL_CXXALIAS_SYS (pthread_cond_init, int,
-                  (pthread_cond_t *cond, const pthread_condattr_t *attr));
+                  (pthread_cond_t *restrict cond,
+                   const pthread_condattr_t *restrict attr));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_cond_init);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_cond_init
 # if HAVE_RAW_DECL_PTHREAD_COND_INIT
@@ -1403,7 +1511,9 @@ _GL_FUNCDECL_SYS (pthread_condattr_init, int, (pthread_condattr_t *attr)
 #  endif
 _GL_CXXALIAS_SYS (pthread_condattr_init, int, (pthread_condattr_t *attr));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_condattr_init);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_condattr_init
 # if HAVE_RAW_DECL_PTHREAD_CONDATTR_INIT
@@ -1428,7 +1538,9 @@ _GL_FUNCDECL_SYS (pthread_condattr_destroy, int, (pthread_condattr_t *attr)
 #  endif
 _GL_CXXALIAS_SYS (pthread_condattr_destroy, int, (pthread_condattr_t *attr));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_condattr_destroy);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_condattr_destroy
 # if HAVE_RAW_DECL_PTHREAD_CONDATTR_DESTROY
@@ -1444,20 +1556,26 @@ _GL_WARN_ON_USE (pthread_condattr_destroy, "pthread_condattr_destroy is not port
 #   define pthread_cond_wait rpl_pthread_cond_wait
 #  endif
 _GL_FUNCDECL_RPL (pthread_cond_wait, int,
-                  (pthread_cond_t *cond, pthread_mutex_t *mutex)
+                  (pthread_cond_t *restrict cond,
+                   pthread_mutex_t *restrict mutex)
                   _GL_ARG_NONNULL ((1, 2)));
 _GL_CXXALIAS_RPL (pthread_cond_wait, int,
-                  (pthread_cond_t *cond, pthread_mutex_t *mutex));
+                  (pthread_cond_t *restrict cond,
+                   pthread_mutex_t *restrict mutex));
 # else
 #  if !@HAVE_PTHREAD_COND_WAIT@
 _GL_FUNCDECL_SYS (pthread_cond_wait, int,
-                  (pthread_cond_t *cond, pthread_mutex_t *mutex)
+                  (pthread_cond_t *restrict cond,
+                   pthread_mutex_t *restrict mutex)
                   _GL_ARG_NONNULL ((1, 2)));
 #  endif
 _GL_CXXALIAS_SYS (pthread_cond_wait, int,
-                  (pthread_cond_t *cond, pthread_mutex_t *mutex));
+                  (pthread_cond_t *restrict cond,
+                   pthread_mutex_t *restrict mutex));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_cond_wait);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_cond_wait
 # if HAVE_RAW_DECL_PTHREAD_COND_WAIT
@@ -1473,24 +1591,30 @@ _GL_WARN_ON_USE (pthread_cond_wait, "pthread_cond_wait is not portable - "
 #   define pthread_cond_timedwait rpl_pthread_cond_timedwait
 #  endif
 _GL_FUNCDECL_RPL (pthread_cond_timedwait, int,
-                  (pthread_cond_t *cond, pthread_mutex_t *mutex,
-                   const struct timespec *abstime)
+                  (pthread_cond_t *restrict cond,
+                   pthread_mutex_t *restrict mutex,
+                   const struct timespec *restrict abstime)
                   _GL_ARG_NONNULL ((1, 2, 3)));
 _GL_CXXALIAS_RPL (pthread_cond_timedwait, int,
-                  (pthread_cond_t *cond, pthread_mutex_t *mutex,
-                   const struct timespec *abstime));
+                  (pthread_cond_t *restrict cond,
+                   pthread_mutex_t *restrict mutex,
+                   const struct timespec *restrict abstime));
 # else
 #  if !@HAVE_PTHREAD_COND_TIMEDWAIT@
 _GL_FUNCDECL_SYS (pthread_cond_timedwait, int,
-                  (pthread_cond_t *cond, pthread_mutex_t *mutex,
-                   const struct timespec *abstime)
+                  (pthread_cond_t *restrict cond,
+                   pthread_mutex_t *restrict mutex,
+                   const struct timespec *restrict abstime)
                   _GL_ARG_NONNULL ((1, 2, 3)));
 #  endif
 _GL_CXXALIAS_SYS (pthread_cond_timedwait, int,
-                  (pthread_cond_t *cond, pthread_mutex_t *mutex,
-                   const struct timespec *abstime));
+                  (pthread_cond_t *restrict cond,
+                   pthread_mutex_t *restrict mutex,
+                   const struct timespec *restrict abstime));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_cond_timedwait);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_cond_timedwait
 # if HAVE_RAW_DECL_PTHREAD_COND_TIMEDWAIT
@@ -1515,7 +1639,9 @@ _GL_FUNCDECL_SYS (pthread_cond_signal, int, (pthread_cond_t *cond)
 #  endif
 _GL_CXXALIAS_SYS (pthread_cond_signal, int, (pthread_cond_t *cond));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_cond_signal);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_cond_signal
 # if HAVE_RAW_DECL_PTHREAD_COND_SIGNAL
@@ -1540,7 +1666,9 @@ _GL_FUNCDECL_SYS (pthread_cond_broadcast, int, (pthread_cond_t *cond)
 #  endif
 _GL_CXXALIAS_SYS (pthread_cond_broadcast, int, (pthread_cond_t *cond));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_cond_broadcast);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_cond_broadcast
 # if HAVE_RAW_DECL_PTHREAD_COND_BROADCAST
@@ -1565,7 +1693,9 @@ _GL_FUNCDECL_SYS (pthread_cond_destroy, int, (pthread_cond_t *cond)
 #  endif
 _GL_CXXALIAS_SYS (pthread_cond_destroy, int, (pthread_cond_t *cond));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_cond_destroy);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_cond_destroy
 # if HAVE_RAW_DECL_PTHREAD_COND_DESTROY
@@ -1593,10 +1723,12 @@ _GL_FUNCDECL_SYS (pthread_key_create, int,
                   (pthread_key_t *keyp, void (*destructor) (void *))
                   _GL_ARG_NONNULL ((1)));
 #  endif
-_GL_CXXALIAS_SYS (pthread_key_create, int,
-                  (pthread_key_t *keyp, void (*destructor) (void *)));
+_GL_CXXALIAS_SYS_CAST (pthread_key_create, int,
+                       (pthread_key_t *keyp, void (*destructor) (void *)));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_key_create);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_key_create
 # if HAVE_RAW_DECL_PTHREAD_KEY_CREATE
@@ -1623,7 +1755,9 @@ _GL_FUNCDECL_SYS (pthread_setspecific, int,
 _GL_CXXALIAS_SYS (pthread_setspecific, int,
                   (pthread_key_t key, const void *value));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_setspecific);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_setspecific
 # if HAVE_RAW_DECL_PTHREAD_SETSPECIFIC
@@ -1646,7 +1780,9 @@ _GL_FUNCDECL_SYS (pthread_getspecific, void *, (pthread_key_t key));
 #  endif
 _GL_CXXALIAS_SYS (pthread_getspecific, void *, (pthread_key_t key));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_getspecific);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_getspecific
 # if HAVE_RAW_DECL_PTHREAD_GETSPECIFIC
@@ -1669,7 +1805,9 @@ _GL_FUNCDECL_SYS (pthread_key_delete, int, (pthread_key_t key));
 #  endif
 _GL_CXXALIAS_SYS (pthread_key_delete, int, (pthread_key_t key));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_key_delete);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_key_delete
 # if HAVE_RAW_DECL_PTHREAD_KEY_DELETE
