@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2020, Free Software Foundation, Inc.
+Copyright (C) 2016-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -157,8 +157,10 @@ gal_data_initialize(gal_data_t *data, void *array, uint8_t type,
       data->size=1;
       for(i=0;i<ndim;++i)
         {
-          /* Size along a dimension cannot be negative. */
-          if(dsize[i] == 0)
+          /* Size along a dimension cannot be 0 if we are in a
+             multi-dimensional dataset. In a single-dimensional dataset, we
+             can have an empty dataset. */
+          if(ndim>1 && dsize[i] == 0)
             error(EXIT_FAILURE, 0, "%s: dsize[%zu]==0. The size of a "
                   "dimension cannot be zero", __func__, i);
 
@@ -188,19 +190,11 @@ gal_data_initialize(gal_data_t *data, void *array, uint8_t type,
         data->array=array;
       else
         {
+          /* If a size wasn't given, just set a NULL pointer. */
           if(data->size)
-            {
-              if( gal_type_sizeof(type)*data->size > minmapsize )
-                /* Allocate the space into disk (HDD/SSD). */
-                data->array=gal_pointer_allocate_mmap(data->type, data->size,
-                                                      clear, &data->mmapname,
-                                                      quietmmap);
-              else
-                /* Allocate the space in RAM. */
-                data->array = gal_pointer_allocate(data->type, data->size,
-                                                   clear, __func__,
-                                                   "data->array");
-            }
+            data->array=gal_pointer_allocate_ram_or_mmap(data->type,
+                                 data->size, clear, minmapsize,
+                                 &data->mmapname, quietmmap, __func__, "");
           else data->array=NULL; /* The given size was zero! */
         }
     }
@@ -245,8 +239,9 @@ gal_data_free_contents(gal_data_t *data)
   if(data->name)    { free(data->name);    data->name    = NULL; }
   if(data->unit)    { free(data->unit);    data->unit    = NULL; }
   if(data->dsize)   { free(data->dsize);   data->dsize   = NULL; }
-  if(data->wcs)     { wcsfree(data->wcs);  data->wcs     = NULL; }
   if(data->comment) { free(data->comment); data->comment = NULL; }
+  if(data->wcs)
+    { wcsfree(data->wcs); free(data->wcs); data->wcs     = NULL; }
 
   /* If the data type is string, then each element in the array is actually
      a pointer to the array of characters, so free them before freeing the
@@ -257,24 +252,14 @@ gal_data_free_contents(gal_data_t *data)
       for(i=0;i<data->size;++i) if(strarr[i]) free(strarr[i]);
     }
 
-  /* Free the array. */
-  if(data->mmapname)
+  /* Free the array (if it was separately allocated: not part of a block),
+     then set the 'array' to NULL. */
+  if(data->array && data->block==NULL)
     {
-      /* Delete the file keeping the array. */
-      remove(data->mmapname);
-
-      /* Inform the user. */
-      if(!data->quietmmap)
-        error(EXIT_SUCCESS, 0, "%s: deleted", data->mmapname);
-
-      /* Free the file name space. */
-      free(data->mmapname);
-
-      /* Set the name pointer to NULL since it has been freed. */
-      data->mmapname=NULL;
+      if(data->mmapname)
+        gal_pointer_mmap_free(&data->mmapname, data->quietmmap);
+      else free(data->array);
     }
-  else
-    if(data->array && data->block==NULL) free(data->array);
   data->array=NULL;
 }
 
@@ -391,6 +376,55 @@ gal_data_array_free(gal_data_t *dataarr, size_t size, int free_array)
 
 
 
+/* Create an array of gal_data_t pointers and initializes them. */
+gal_data_t **
+gal_data_array_ptr_calloc(size_t size)
+{
+  size_t i;
+  gal_data_t **out;
+
+  /* Allocate the array to keep the pointers. */
+  errno=0;
+  out=malloc(size*sizeof *out);
+  if(out==NULL)
+    error(EXIT_FAILURE, errno, "%s: %zu bytes for 'out'", __func__,
+          size*sizeof *out);
+
+  /* Initialize all the pointers to NULL and return. */
+  for(i=0;i<size;++i) out[i]=NULL;
+  return out;
+}
+
+
+
+
+
+/* Assuming that we have an array of pointers to data structures, this
+   function frees them. */
+void
+gal_data_array_ptr_free(gal_data_t **dataptr, size_t size, int free_array)
+{
+  size_t i;
+  for(i=0;i<size;++i)
+    {
+      /* If the user doesn't want to free the array, it must be because
+         they are keeping its pointer somewhere else (that their own
+         responsability!), so we can just set it to NULL for the
+         'gal_data_free' to not free it. */
+      if(free_array==0)
+        dataptr[i]->array=NULL;
+
+      /* Free this data structure. */
+      gal_data_free(dataptr[i]);
+    }
+
+  /* Free the 'gal_data_t **'. */
+  free(dataptr);
+}
+
+
+
+
 
 
 
@@ -801,32 +835,34 @@ gal_data_copy_to_allocated(gal_data_t *in, gal_data_t *out)
   gal_checkset_allocate_copy(in->comment, &out->comment);
 
   /* Do the copying. */
-  switch(out->type)
-    {
-    case GAL_TYPE_UINT8:   COPY_OT_SET( uint8_t  );      break;
-    case GAL_TYPE_INT8:    COPY_OT_SET( int8_t   );      break;
-    case GAL_TYPE_UINT16:  COPY_OT_SET( uint16_t );      break;
-    case GAL_TYPE_INT16:   COPY_OT_SET( int16_t  );      break;
-    case GAL_TYPE_UINT32:  COPY_OT_SET( uint32_t );      break;
-    case GAL_TYPE_INT32:   COPY_OT_SET( int32_t  );      break;
-    case GAL_TYPE_UINT64:  COPY_OT_SET( uint64_t );      break;
-    case GAL_TYPE_INT64:   COPY_OT_SET( int64_t  );      break;
-    case GAL_TYPE_FLOAT32: COPY_OT_SET( float    );      break;
-    case GAL_TYPE_FLOAT64: COPY_OT_SET( double   );      break;
-    case GAL_TYPE_STRING:  data_copy_to_string(in, out); break;
+  if(in->array)
+    switch(out->type)
+      {
+      case GAL_TYPE_UINT8:   COPY_OT_SET( uint8_t  );      break;
+      case GAL_TYPE_INT8:    COPY_OT_SET( int8_t   );      break;
+      case GAL_TYPE_UINT16:  COPY_OT_SET( uint16_t );      break;
+      case GAL_TYPE_INT16:   COPY_OT_SET( int16_t  );      break;
+      case GAL_TYPE_UINT32:  COPY_OT_SET( uint32_t );      break;
+      case GAL_TYPE_INT32:   COPY_OT_SET( int32_t  );      break;
+      case GAL_TYPE_UINT64:  COPY_OT_SET( uint64_t );      break;
+      case GAL_TYPE_INT64:   COPY_OT_SET( int64_t  );      break;
+      case GAL_TYPE_FLOAT32: COPY_OT_SET( float    );      break;
+      case GAL_TYPE_FLOAT64: COPY_OT_SET( double   );      break;
+      case GAL_TYPE_STRING:  data_copy_to_string(in, out); break;
 
-    case GAL_TYPE_BIT:
-    case GAL_TYPE_STRLL:
-    case GAL_TYPE_COMPLEX32:
-    case GAL_TYPE_COMPLEX64:
-      error(EXIT_FAILURE, 0, "%s: copying to %s type not yet supported",
-            __func__, gal_type_name(out->type, 1));
-      break;
+      case GAL_TYPE_BIT:
+      case GAL_TYPE_STRLL:
+      case GAL_TYPE_COMPLEX32:
+      case GAL_TYPE_COMPLEX64:
+        error(EXIT_FAILURE, 0, "%s: copying to %s type not yet supported",
+              __func__, gal_type_name(out->type, 1));
+        break;
 
-    default:
-      error(EXIT_FAILURE, 0, "%s: type %d not recognized for 'out->type'",
-            __func__, out->type);
-    }
+      default:
+        error(EXIT_FAILURE, 0, "%s: type %d not recognized for 'out->type'",
+              __func__, out->type);
+      }
+  else out->array=NULL;
 
   /* Correct the sizes of the output to be the same as the input. If it is
      equal, there is no problem, if not, the size information will be
