@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2019, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -26,7 +26,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 #define GAL_STRINGS_SHORT_COPYRIGHT                                     \
-  "Copyright (C) 2015-2019, Free Software Foundation, Inc."
+  "Copyright (C) 2015-2021, Free Software Foundation, Inc."
 
 
 #define GAL_STRINGS_SHORT_LICENSE                                       \
@@ -64,7 +64,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* This can be used in the end of error messages related to option
    values. */
 #define GAL_STRINGS_HOW_TO_CHECK_VALUES                                 \
-  " You can check all the input values with the `--printparams' "       \
+  " You can check all the input values with the '--printparams' "       \
   "(-P) option."
 
 
