@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -85,6 +85,12 @@ struct wcsprm *
 gal_wcs_read(char *filename, char *hdu, size_t hstartwcs,
              size_t hendwcs, int *nwcs);
 
+struct wcsprm *
+gal_wcs_create(double *crpix, double *crval, double *cdelt,
+               double *pc, char **cunit, char **ctype, size_t ndim);
+
+char *
+gal_wcs_dimension_name(struct wcsprm *wcs, size_t dimension);
 
 
 
@@ -152,7 +158,10 @@ gal_wcs_pixel_scale(struct wcsprm *wcs);
 double
 gal_wcs_pixel_area_arcsec2(struct wcsprm *wcs);
 
-
+int
+gal_wcs_coverage(char *filename, char *hdu, size_t *ndim,
+                 double **center, double **width, double **min,
+                 double **max);
 
 
 
