@@ -4,7 +4,7 @@ Function to parse options and configuration file values.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2020, Free Software Foundation, Inc.
+Copyright (C) 2017-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -898,6 +898,11 @@ gal_options_parse_csv_strings(struct argp_option *option, char *arg,
       /* If the option is already set, just return. */
       if(option->set) return NULL;
 
+      /* Make sure an argument is actually given. */
+      if(*arg=='\0')
+        error_at_line(EXIT_FAILURE, 0, filename, lineno, "no value "
+                      "given to '--%s'", option->name);
+
       /* Read the values. */
       values=gal_options_parse_csv_strings_raw(arg, filename, lineno);
 
@@ -911,6 +916,61 @@ gal_options_parse_csv_strings(struct argp_option *option, char *arg,
 
 
 
+/* Some options can be called multiple times on the command-line, but
+   within the program, all the values must be merged into one. For example
+   '--column=1 --column=2,3 --column=4,5,6'. In this example, the output
+   'gal_list_str_t' will have 3 nodes, but we actually want a list that has
+   6 nodes. */
+void
+gal_options_merge_list_of_csv(gal_list_str_t **list)
+{
+  size_t i;
+  gal_data_t *strs;
+  char *c, **strarr;
+  gal_list_str_t *tmp, *in=*list, *out=NULL;
+
+  /* Go over each input and add it to the list. */
+  for(tmp=in; tmp!=NULL; tmp=tmp->next)
+    {
+      /* Remove any possibly commented new-line where we have a backslash
+         followed by a new-line character (replace the two characters with
+         two single space characters). This can happen with options have
+         have longer scripts and the user is forced to break the line with
+         a '\' followed by newline. */
+      for(c=tmp->v;*c!='\0';++c)
+        if(*c=='\\' && *(c+1)=='\n') { *c=' '; *(++c)=' '; }
+
+      /* Read the different comma-separated strings into an array (within a
+         'gal_data_t'). */
+      strs=gal_options_parse_csv_strings_raw(tmp->v, NULL, 0);
+      strarr=strs->array;
+
+      /* Go through all the items and add the pointers to the output
+         list. We won't re-allocate the string, we'll just set it to NULL
+         in the array. */
+      for(i=0;i<strs->size;++i)
+        {
+          gal_list_str_add(&out, strarr[i], 0);
+          strarr[i]=NULL;
+        }
+
+      /* Clean up. */
+      gal_data_free(strs);
+    }
+
+  /* Free the input list and reverse the output list to be in the same
+     input order. */
+  gal_list_str_free(in, 1);
+  gal_list_str_reverse(&out);
+
+  /* Reset the input pointer. */
+  *list=out;
+}
+
+
+
+
+
 /* Parse the given string into a series of size values (integers, stored as
    an array of size_t). The output array will be stored in the 'value'
    element of the option. The last element of the array is
@@ -960,6 +1020,11 @@ gal_options_parse_sizes_reverse(struct argp_option *option, char *arg,
       /* If the option is already set, just return. */
       if(option->set) return NULL;
 
+      /* Make sure an argument is actually given. */
+      if(*arg=='\0')
+        error_at_line(EXIT_FAILURE, 0, filename, lineno, "no value "
+                      "given to '--%s'", option->name);
+
       /* Read the values. */
       values=gal_options_parse_list_of_numbers(arg, filename, lineno);
 
@@ -1042,6 +1107,11 @@ gal_options_parse_csv_float64(struct argp_option *option, char *arg,
       /* If the option is already set, just return. */
       if(option->set) return NULL;
 
+      /* Make sure an argument is actually given. */
+      if(*arg=='\0')
+        error_at_line(EXIT_FAILURE, 0, filename, lineno, "no value "
+                      "given to '--%s'", option->name);
+
       /* Read the values. */
       values=gal_options_parse_list_of_numbers(arg, filename, lineno);
 
@@ -1183,6 +1253,11 @@ gal_options_parse_name_and_values(struct argp_option *option, char *arg,
     }
   else
     {
+      /* Make sure an argument is actually given. */
+      if(*arg=='\0')
+        error_at_line(EXIT_FAILURE, 0, filename, lineno, "no value "
+                      "given to '--%s'", option->name);
+
       /* Parse until the comma or the end of the string.*/
       c=arg; while(*c!='\0' && *c!=',') ++c;
       values = (*c=='\0') ? NULL : c+1;
@@ -1399,6 +1474,11 @@ gal_options_parse_colon_sep_csv(struct argp_option *option, char *arg,
     }
   else
     {
+      /* Make sure an argument is actually given. */
+      if(*arg=='\0')
+        error_at_line(EXIT_FAILURE, 0, filename, lineno, "no value "
+                      "given to '--%s'", option->name);
+
       /* Parse the desired format and put it in this option's pointer. */
       dataset=options_parse_colon_sep_csv(arg, filename, lineno);
 
@@ -2371,6 +2451,8 @@ option_is_printable(struct argp_option *option)
     case GAL_OPTIONS_KEY_LASTCONFIG:
       return 0;
     }
+
+  /* Everything is fine, print the option. */
   return 1;
 }
 
@@ -2788,6 +2870,7 @@ options_as_fits_keywords_write(gal_fits_list_key_t **keys,
 {
   size_t i;
   void *vptr;
+  int vptrfree;
   uint8_t vtype;
   char *name, *doc;
   gal_list_str_t *tmp;
@@ -2803,8 +2886,8 @@ options_as_fits_keywords_write(gal_fits_list_key_t **keys,
               /* 'name' and 'doc' have a 'const' qualifier. */
               gal_checkset_allocate_copy(options[i].name, &name);
               gal_checkset_allocate_copy(options[i].doc,  &doc);
-              gal_fits_key_list_add(keys, GAL_TYPE_STRING, name, 1, tmp->v,
-                                    0, doc, 1, NULL, 0);
+              gal_fits_key_list_add(keys, GAL_TYPE_STRING, name, 1,
+                                    tmp->v, 0, doc, 1, NULL, 0);
             }
         /* Normal types. */
         else
@@ -2812,11 +2895,13 @@ options_as_fits_keywords_write(gal_fits_list_key_t **keys,
             /* If the option is associated with a special function for
                reading and writing, we'll need to write the value as a
                string. */
+            vptrfree=0;
             if(options[i].func)
               {
+                vptrfree=1;
                 vtype=GAL_TYPE_STRING;
-                vptr=options[i].func(&options[i], NULL, NULL, (size_t)(-1),
-                                      cp->program_struct);
+                vptr=options[i].func(&options[i], NULL, NULL,
+                                     (size_t)(-1), cp->program_struct);
               }
             else
               {
@@ -2834,8 +2919,8 @@ options_as_fits_keywords_write(gal_fits_list_key_t **keys,
             else
               {
                 gal_checkset_allocate_copy(options[i].doc,  &doc);
-                gal_fits_key_list_add(keys, vtype, name, 1, vptr, 0, doc, 1,
-                                      NULL, 0);
+                gal_fits_key_list_add(keys, vtype, name, 1, vptr,
+                                      vptrfree, doc, 1, NULL, 0);
               }
           }
       }
